package revnetwork;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.*;

public class ReversiNetwork {

	private static byte HEADER_MOVE = 1;
	private static byte HEADER_LOGIN = 2;
	private static byte HEADER_MESSAGE = 5;
	
	private static int PACKET_SIZE = 1024;

	private static String serverAddress = "192.168.1.102";
	private static int serverPort = 1234;
	private static Socket socketServer;

	public ReversiNetwork(String server, int port) {
		serverAddress = server;
		serverPort = port;
	}


	public int connectToServer(String localInterface, int localPort) {
		try {
			InetAddress inetServer = InetAddress.getByName(serverAddress);
			InetAddress inetClient = InetAddress.getByName(localInterface);
			socketServer = new Socket(inetServer, serverPort, inetClient,
					localPort);
			System.out.println(socketServer);

		} catch (UnknownHostException e) {
			return 0;
		} catch (IOException e) {
			return 0;
		}
		return 1;
	}

	public int connectToServerSimple() {
		try {
			InetAddress inetServer = InetAddress.getByName(serverAddress);
			socketServer = new Socket(inetServer, serverPort);
			System.out.println(socketServer);

		} catch (UnknownHostException e) {
			return 0;
		} catch (IOException e) {
			return 0;
		}
		return 1;
	}

	/*
	public static int pingServer() {
		PingThread pinger = new PingThread(pingOffset, socketServer,
				HEADER_PING);
		pinger.start();
		return 0;
	}*/

	public int sendMoveToServer(int x, int y) throws IOException {
		byte bx = (byte) x;
		byte by = (byte) y;
		byte[] toSend = new byte[PACKET_SIZE];
		for (int i=0;i<PACKET_SIZE;i++) {
			toSend[i] = 0;
		}
		toSend[0] = HEADER_MOVE;
		toSend[1] = bx;
		toSend[2] = by;
		DataOutputStream output = new DataOutputStream(socketServer
				.getOutputStream());
		output.write(toSend);

		return 0;
	}

	public void sendMessage(String message) throws IOException {
		char[] cdata = new char[message.length()];
		byte[] bdata = new byte[message.length() + 1];
		bdata[0] = HEADER_MESSAGE;
		cdata = message.toCharArray();
		DataOutputStream output = new DataOutputStream(socketServer
				.getOutputStream());
		for (int i = 0; i < message.length(); i++) {
			bdata[i + 1] = (byte) cdata[i];
		}
		output.write(bdata);
	}

	public void sendData(byte[] data) throws IOException {
		DataOutputStream output = new DataOutputStream(socketServer
				.getOutputStream());
		output.write(data);
	}

	public void sendLogin(String login) throws IOException {
		char[] cdata = new char[login.length()];
		byte[] bdata = new byte[PACKET_SIZE];
		
		cdata = login.toCharArray();
		DataOutputStream output = new DataOutputStream(socketServer
				.getOutputStream());
		for (int i = 0; i < PACKET_SIZE; i++) {
			bdata[i]=0;
		}
		bdata[0] = HEADER_LOGIN;
		for (int i = 0; i < login.length(); i++) {
			bdata[i + 1] = (byte) cdata[i];
		}
		
		output.write(bdata);
	}

	public int[][] getMatrixFromServer() {
		byte[] bmatrix = new byte[PACKET_SIZE];
		int[][] imatrix = new int[10][10];
		try {
			DataInputStream in = new DataInputStream(socketServer
					.getInputStream());
			//in.read(bmatrix);
			in.read(bmatrix, 0, 100);
			for (int i = 0; i < 100; i++) {
				imatrix[i % 10][i / 10] = (bmatrix[i]);
				System.out.print(imatrix[i%10][i/10] + " ");
				if (i%10==9) {
					System.out.println();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return imatrix;
	}

	public byte getInstruction() throws IOException {
			byte[] inst = new byte[1];
			DataInputStream in = new DataInputStream(socketServer
					.getInputStream());
			if (in == null) {
				return -1;
			}
			in.read(inst, 0, 1);
			return inst[0];
	}
	
	@SuppressWarnings("finally")
	public StringBuffer getMessage() {
		StringBuffer buf = new StringBuffer("");
		DataInputStream in;
		try {
			in = new DataInputStream(socketServer.getInputStream());

		byte[] temp = new byte[PACKET_SIZE];
		int read;
		read = in.read(temp);
		for (int i=0; i<read; i++) {
			if (temp[i] != 0) {
				buf.append((char)(temp[i]));
			}
		}
		} catch (IOException e) {
			e.printStackTrace();
		}
		finally {
			return buf;
		}
	}

	@SuppressWarnings("finally")
	public String[] getTopScore() {
		StringBuffer buf = new StringBuffer();
		String[] top = new String[20]; 
		
		try {
			DataInputStream in = new DataInputStream(socketServer
					.getInputStream());
			byte[] temp = new byte[PACKET_SIZE];
			System.out.println("TOPSCORE READ: " + in.read(temp,0,PACKET_SIZE-1));
			int i = 0;
			int j = 0;

			while (i<PACKET_SIZE-1) {
				
				if (temp[i]==0) {
					while(temp[i]==0) {
						i++;
					}
				}
				
				if (temp[i]==9) {
					top[j]=buf.toString();
					buf.delete(0, buf.length());
					j++;
					i++;
				}
				
				if (temp[i] == 10) {
					top[j]=buf.toString();
					buf.delete(0, buf.length());
					j++;
					i++;
				}
				buf.append((char)temp[i]);
				i++;

			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			return top;
		}
	}
	
	@SuppressWarnings("finally")
	public int getTurn() {
		byte[] temp = new byte[PACKET_SIZE-101];
		int last = -1;
		try {
			DataInputStream in = new DataInputStream(socketServer
					.getInputStream());
			last = in.read(temp,0,923);
			System.out.println("last" + last);
			
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			
			return temp[last-1];
		}
	}
	
	public void cleanInput(int length) {
		byte[] temp = new byte[length+1];
		try {
			DataInputStream in = new DataInputStream(socketServer
					.getInputStream());
			in.read(temp,0,length);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public int disconnectFromServer()  {
		if (socketServer!=null) {
			if (socketServer.isClosed()) {
				return 1;
			} else {
				try {
				byte[] data = new byte[1024];
				for (int i=0;i<1024;i++) {
					data[i]=0;
				}
				data[0]=(byte)9;
				this.sendData(data);
				socketServer.shutdownInput();
				socketServer.shutdownOutput();
				socketServer.close();
				} catch (IOException e) {
					System.out.println("Polaczenie nie moze zostac zakonczone");
				}
			}
			if (socketServer.isClosed()) {
				return 1;
			} else {
				return 0;
			}
		} else {
			return 0;
		}

	}

	/**
	 * @param args
	 */
	/*
	 * public static void main(String[] args) { //showInterfaces();
	 * connectToServer("150.254.141.56",5555); //pingServer();
	 * sendMoveToServer(122, 9); //getMatrixFromServer();
	 * //disconnectFromServer(); System.out.println("REVERSI NETWORK CLASS");
	 * 
	 * }
	 */
}

