 /*
 ipcs|tr -s ' ' '\t'|cut -f2|xargs -n1 ipcrm -q
 */

//obsluga socketow
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <netdb.h>

//standardowe
#include <stdio.h>
#include <stdlib.h>

//czas
#include <time.h>

//procesy : fork, wait, getpid, getppid
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
//sygnaly
#include <signal.h>

//kolejki komunikatow
#include <sys/msg.h>
//pamiec wspoldzielona
#include <sys/ipc.h>
#include <sys/shm.h>
//semafory
#include <sys/sem.h>
//operacje na lancuchach
#include <string.h>
//operacje na plikach
#include <fcntl.h>


//port na którym działa serwer
#define PORT htons(4321)
//rozmiar bufora
#define BUF_SIZE 1024
//rozmiar kolejki docierających żądań połączeń (f-cja listen)
#define BACKLOG 10
//prawa do kolejek, pamieci wspoldzielonej
#define PERM 0777
//klucz kolejki
#define KEY 8000
//maksymalna liczba graczy
#define MAX_USR 10
//maksymalna liczba utworzonych gniazdek
#define SOC_NMB 11
//timeout do f-cji select
#define TIMEOUT 5
//rozmiar planszy
#define BOARD_SIZE 10
//maksymalna  dlugosc nicka
#define NICK_LENGTH 10
//maksymalna ilosc cyfer w top score
#define POINTS_LENGTH 3
//ilosc najlepszych wynikow w top score
#define TS 10
//wyswietlanie komunikatow na serwerze 0 - tak ; 1 nie
#define DSP_MSG 0
//naglowek przesylany podczas logowania
#define LOG_HEAD 10
//naglowek przesylany podczas gry
#define GAME_HEAD 1
//naglowek przesylany po zakonczeniu rozgrywki
#define END_HEAD 2
//naglowek do przesylania top scora
#define TS_HEAD 3
//naglowek przesylany podczas czatu
#define CHAT_HEAD 5
//naglowek przesylany gdy komunikacja z panelem administratora
#define WM_HEAD 8
//naglowek przesylany kiedy administrator chce zrestartowac serwer
#define WM_RESTART 7
//naglowek przesylany kiedy administrator chce zobaczyc aktualna liczbe uzytkownikow
#define WM_NMBUSR 6
//naglowek przesylany kiedy administrator konczy prace
#define WM_END 5
//naglowek przesylany kiedy administartor chce sprawdzic maksymalna liczbe uzytkownikow
#define WM_MAXUSR 4
//naglowek przesylany w razie wystapienia bledu
#define ERR_HEAD 9
//naglowek przesylany miedzy watkami podczas gry
#define GAME_TH 0
//naglowek przesylany miedzy watkiemi podczas czatu
#define CHAT_TH 1
//naglowek przesylany miedzy watkiami na zakonczenie rozgrywki
#define END_TH 2
//naglowek przesylany w przypadku gdy gracz wykona nieprawidlowy ruch
#define REPEAD_TH 3
//naglowek przesylany miedzy watkiami w przypadku bledu
#define ERR_TH 9
//tolerancja czasowa - maksymalny czas jaki moga istniec watki
#define TOLERATION 1




int msgq_id;		//identyfikator kolejki komunikatow odpowiedzialnej z komunikowanie sie watkow na serwerze

int shmid_pid;		//identyfikator segmentu pamieci wspoldzielonej ktora trzyma pidy watkow wszyskich graczy
int shmid_usr;		//identyfikator segmentu pamieci wspoldzielonej dla aktualnej liczby uzytkownikow w systemie
int shmid_mlp;		//identyfikator segmentu pamieci wspoldzielonej w ktorej znajduja pidy uzytkownikow grajacych w trybie multiplayer
int shmid_soc;		//identyfikator segmentu pamieci wspoldzielonej przechowujacej deskryptory wszyskich otwartych gniazdek
int shmid_tsc;		//identyfikator segmentu pamieci wspoldzielonej do przechowywania najlepszych wynikow
int shmid_val;		//identyfikator segmentu pamieci wspoldzielonej odpowiedzialny za dlugosc zycia watkow


int semid_usr;		//identyfikator semafora odpowiedzialnego za pamiec z liczba uzytkownikow


struct sembuf P;	//struktura opisujaca opuszczenie semafora dla shmid_usr
struct sembuf V;	//struktura opisujaca podniesienie semafora dla shmid_usr

struct sembuf Ppid;	//opuszczenie semafora dla shmid_pid
struct sembuf Vpid;	//podniesienie semafora dla shmid_pid

struct sembuf Pmlp;	//opuszczenie semafora dla shmid_mlp
struct sembuf Vmlp;	//podniesienie semafora dla shmid_mlp

struct sembuf Psoc;	//opuszczenie semafora dla shmid_soc
struct sembuf Vsoc;	//podniesienie semafora dla shmid_soc

struct sembuf Ptsc;	//opuszczenie semafora dla shmid_tsc
struct sembuf Vtsc;	//podniesienie semafora dla shmid_tsc

struct sembuf Pval;	//opuszczenie semafora dla shmid_val
struct sembuf Vval;	//podniesienie semafora dla shmid_val


struct message		//struktura opisujaca wiadomosc przesylana miedzy watkami
{
	long type;		//typ komunikatu
	char content[BUF_SIZE];	//zawartosc komunikatu
};

int msg_size = sizeof(struct message) - sizeof(long);

struct top_score
{
	char nick[NICK_LENGTH];	        //maksymalna dlugosc nazwy uzytkownika
	char points[POINTS_LENGTH];	//maksymalna liczba punktow 999
};

struct validate
{
	int log_time;
	int pid_thr;
	int dsc_thr;
};

void create_queue()
{
	msgq_id = msgget(KEY, IPC_CREAT|IPC_EXCL|PERM);
	if(msgq_id == -1)
	{
		perror("blad podczas tworzenia kolejki");
		exit(-1);
	}

	printf("tworzenie kolejek:\t\t\tOK\n");
}

void remove_queue()
{
	int ret_val=0;

	ret_val = msgctl(msgq_id, IPC_RMID,0);
	if(ret_val == -1)
		perror("blad podczas usuwania kolejek");

	printf("usuwanie kolejek:\t\t\tOK\n");
}

void create_shared_memory()		//tworzy pamiec wspoldzielona
{
	int i=0;
	int* shm_addr = NULL;


	shmid_pid = shmget(IPC_PRIVATE, sizeof(int [MAX_USR]), IPC_CREAT|PERM);			//w tablicy przetrzymywane beda pidy wszystkich watkow klientow
	if(shmid_pid == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej pid");
		exit(-1);
	}

	shm_addr = shmat(shmid_pid, NULL, 0);
	for(i = 0 ; i < (MAX_USR) ; i++)
	{
		shm_addr[i] = 0;
	}
	shmdt(shm_addr);


	shmid_usr = shmget(IPC_PRIVATE, sizeof(int), IPC_CREAT|PERM);			//zmienna do kontroli liczby uzytkownikow w systemie
	if(shmid_usr == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej dla liczby uzytkonikow");
		exit(-1);
	}

	shm_addr = shmat(shmid_usr, NULL, 0);
	*shm_addr = 0;
	shmdt(shm_addr);


	shmid_mlp = shmget(IPC_PRIVATE, sizeof(int[MAX_USR * 2]), IPC_CREAT|PERM);			//zapamietywane sa pidy uzytkownikow grajacych w trybie multiplayer
	if(shmid_mlp == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej dla mlp");
		exit(-1);
	}

	shm_addr = shmat(shmid_mlp, NULL, 0);
	for(i = 0 ; i < (MAX_USR) ; i++)
	{
		shm_addr[i] = 0;
	}
	shmdt(shm_addr);

	shmid_soc = shmget(IPC_PRIVATE, sizeof(int[SOC_NMB]), IPC_CREAT|PERM);			//zapamietywane sa deskryptory wszystkich gniazdek (maksymalnie w systemie powinno byc ich tyle ile MAX_USR + deskryptor serwera
	if(shmid_soc == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej dla soc");
		exit(-1);
	}

	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		shm_addr[i] = 0;
	}
	shmdt(shm_addr);

	shmid_tsc = shmget(IPC_PRIVATE, sizeof(struct top_score[TS]), IPC_CREAT|PERM);
	if(shmid_tsc == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej dla top score");
		exit(-1);
	}
	struct top_score* ptr;
	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);
	if(ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej dla top score");
		exit(-1);
	}
	int j=0;
	for(i = 0 ; i < TS ; i++)
	{
		j=0;
		while(j < NICK_LENGTH)
		{
			ptr->nick[j] = 0;
			j++;
		}

		j=0;
		while(j < POINTS_LENGTH)
		{
			ptr->points[j] = 0;			//punkty do top scora w pamieci wspoldzielonej przechowywane sa jako char np 032 (3 komorki)
			j++;
		}

		ptr++;
	}
	shmdt(ptr);



	shmid_val = shmget(IPC_PRIVATE, sizeof(struct validate[MAX_USR]), IPC_CREAT|PERM);
	if(shmid_val == -1)
	{
		perror("Blad podczas tworzenia pamieci wspoldzielonej dla validate");
		exit(-1);
	}
	struct validate* v_ptr;
	v_ptr = (struct validate*)shmat(shmid_val, NULL, 0);
	if(v_ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej dla validate");
		exit(-1);
	}

	for(i = 0 ; i < MAX_USR ; i++)
	{
		v_ptr->dsc_thr = 0;
		v_ptr->log_time = 0;
		v_ptr->pid_thr = 0;

		v_ptr++;
	}
	shmdt(v_ptr);

	printf("tworzenie pamieci wspoldzielonej:\tOK\n");
}

void remove_shared_memory()
{
	shmctl(shmid_pid, IPC_RMID,0);
	shmctl(shmid_usr, IPC_RMID,0);
	shmctl(shmid_mlp, IPC_RMID,0);
	shmctl(shmid_soc, IPC_RMID,0);
	shmctl(shmid_tsc, IPC_RMID,0);
	shmctl(shmid_val, IPC_RMID,0);

	printf("usuwanie pamieci wspoldzielonej:\tOK\n");
}

void create_semafor()
{
	P.sem_num = 0;
	P.sem_op = -1;
	P.sem_flg = 0;

	V.sem_num = 0;
	V.sem_op = 1;
	V.sem_flg = 0;


	Ppid.sem_num = 1;
	Ppid.sem_op = -1;
	Ppid.sem_flg = 0;

	Vpid.sem_num = 1;
	Vpid.sem_op = 1;
	Vpid.sem_flg = 0;


	Pmlp.sem_num = 2;
	Pmlp.sem_op = -1;
	Pmlp.sem_flg = 0;

	Vmlp.sem_num = 2;
	Vmlp.sem_op = 1;
	Vmlp.sem_flg = 0;


	Psoc.sem_num = 3;
	Psoc.sem_op = -1;
	Psoc.sem_flg = 0;

	Vsoc.sem_num = 3;
	Vsoc.sem_op = 1;
	Vsoc.sem_flg = 0;


	Ptsc.sem_num = 4;
	Ptsc.sem_op = -1;
	Ptsc.sem_flg = 0;

	Vtsc.sem_num = 4;
	Vtsc.sem_op = 1;
	Vtsc.sem_flg = 0;

	Pval.sem_num = 5;
	Pval.sem_op = -1;
	Pval.sem_flg = 0;

	Vval.sem_num = 5;
	Vval.sem_op = 1;
	Vval.sem_flg = 0;


	semid_usr = semget(IPC_PRIVATE, 6, IPC_CREAT|0644);
	if(semid_usr == -1)
	{
		perror("blad podczas tworzenia semafora");
		exit(-1);
	}
	semop(semid_usr, &V, 1);		//semafory trzeba zainicjowac wywolujac operacje podniesienia
	semop(semid_usr, &Vpid, 1);
	semop(semid_usr, &Vmlp, 1);
	semop(semid_usr, &Vsoc, 1);
	semop(semid_usr, &Vtsc, 1);
	semop(semid_usr, &Vval, 1);

	printf("tworzenie semaforow:\t\t\tOK\n");
}

void remove_semafor()
{
	semctl(semid_usr, 0, IPC_RMID);
	semctl(semid_usr, 1, IPC_RMID);
	semctl(semid_usr, 2, IPC_RMID);
	semctl(semid_usr, 3, IPC_RMID);
	semctl(semid_usr, 4, IPC_RMID);
	semctl(semid_usr, 5, IPC_RMID);
	printf("usuwanie semaforow:\t\t\tOK\n");
}
/*
void clean_buffer(char* buf)
{
	int counter=0;
	while(counter < BUF_SIZE)
	{
		buf[counter] = '';
	}
}
*/


void kill_all_threads()
{
	int* shmaddr_pid;
	int i=0, ret_val=0;

	shmaddr_pid = shmat(shmid_pid, NULL,0);
	if(shmaddr_pid == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w clean");
		exit(-1);
	}

	for(i = 0 ; i < MAX_USR ; i++)				//zabijamy
	{
		if(shmaddr_pid[i] != 0)
		{
			ret_val = kill(shmaddr_pid[i], 9);
			if(ret_val == -1)
			{
				if(DSP_MSG == 0)
				{
					printf("nie udalo sie zabic wszysktkich watku: %d\n", shmaddr_pid[i]);
				}
			}
			else
			{
				if(DSP_MSG == 0)
				{
					printf("zabito watek: %d\n", shmaddr_pid[i]);
				}
			}
		}
	}

	if(DSP_MSG == 0)
	{
		printf("wyskoczyl\n");
	}
	shmdt(shmaddr_pid);
}



int who_is_winner(char board[][BOARD_SIZE], int player)		//zwraca 0 gdy player przegral; 1 gdy wygral
{
	int i=0,j=0;
	int my_points=0;

	for(i = 0 ; i < BOARD_SIZE ; i++)
	{
		for(j = 0 ; j < BOARD_SIZE ; j++)
		{
			if(board[i][j] == player)
			{
				my_points++;
			}
		}
	}

	return my_points;
}

void get_top_score()				//wczytuje z pliku tpsc.cfg do pamieci wspoldzielonej shmid_tsc
{
	if(DSP_MSG == 0)
	{
		printf("\nget top score\n");
	}

	struct top_score* ptr;	//wskaznik do pamieci wspoldzielonej gdzie ma byc zapisany top score
	int fd;			//deskryptor do pliku z top scorem
	char buf=' ';
	int len,flg=0,i=0,j=0;


	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);
	if(ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w top score");
		exit(-1);
	}

	fd = open("tpsc.cfg", O_RDONLY);
	if( fd == -1)
	{
		perror("Blad podczas odczytu z pliku");
		exit(-1);
	}
	i=0;
	j=0;
	flg=0;
	int count=0;
	while( ((len = read(fd, &buf, sizeof(char)) ) != 0 ) )
	{
		//printf("%c", buf);

		if(buf == '\n')
		{
			printf("\n");
			flg = 0;
			i = 0;
			j = 0;
			ptr++;
		}
		else
		{
			if(buf == '\t')
			{
				flg++;
			}
			else
			{
				if((i < NICK_LENGTH) && (flg == 0) )
				{
					if(DSP_MSG == 0)
					{
						printf("%c#", buf);
					}
					ptr->nick[i] = buf;
					i++;
				}
				else
				{
					if(j < POINTS_LENGTH)
					{
						if(DSP_MSG == 0)
						{
							printf("%c$", buf);
						}
						ptr->points[j] = buf;
						j++;
					}
				}
			}

		}

		buf = ' ';

		count++;

	}
	close(fd);
	shmdt(ptr);	//odlaczamy pamiec wspoldzielona

	/*
	printf("\n++++++++++++++\n");
	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);
	for(i = 0 ; i < TS ; i++)
	{
		printf("nick %c\tpoints %c\n", ptr->nick[0], ptr->points[0]);
		ptr++;

	}
	shmdt(ptr);	//odlaczamy pamiec wspoldzielona

	printf("\n++++++++++++++\n");
	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);
	for(i = 0 ; i < TS ; i++)
	{
		printf("nick %c\tpoints %c\n", ptr->nick[1], ptr->points[1]);
		ptr++;

	}
	shmdt(ptr);	//odlaczamy pamiec wspoldzielona
	*/
	/*
		for(i = 0 ; i < TS ; i++)
	{
		ptr->nick[0] = '0';
		ptr->points[0] = '0';
		ptr++;
	}
	shmdt(ptr)
	*/
}

void save_top_score()                           //zapisuje top score do pliku
{
	if(DSP_MSG == 0)
	{
		printf("\nsave top score\n");
	}

	struct top_score* ptr; //wskaznik do pamieci wspoldzielonej gdzie ma byc zapisany top score
	int fd;                //deskryptor do pliku
	int i=0,j=0;
	char tab='\t';
	char nl='\n';
	char value=1;


	fd = open("tpsc.cfg", O_WRONLY|O_TRUNC);		//uwaga musi byc trunc (obciecie pliku bo inaczej zostawiane sa znaki z poprzedniego pliku)
	if(fd == -1)
	{
		perror("blad podczas otwierania pliku do zapisu");
		exit(-1);


	}

	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);    //przylaczenie pamieci wspoldzielonej
	if(ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w save_top_score");
		exit(-1);
	}

	for(i = 0 ; i < TS ; i++)
	{
		//for(j=0; j < NICK_LENGTH; j++)
		j=0;
		value=1;
		while(j < NICK_LENGTH)
		{
			if(ptr->nick[j] == 0)
				break;
			value = ptr->nick[j];
			if(DSP_MSG == 0)
			{
				printf("%c",value);
			}
			write(fd, &value, sizeof(char));
			j++;
		}

		write(fd, &tab, sizeof(char));

		for(j=0; j < POINTS_LENGTH; j++)
		{
			value = ptr->points[j];
			if(DSP_MSG == 0)
			{
				printf("%c",value);
			}
			write(fd, &value, sizeof(char));
		}

		if(i < (TS-1))
			write(fd, &nl, sizeof(char));
		ptr++;
	}
	close(fd);
	shmdt(ptr);    //odlaczamy pamiec wspoldzielona

	printf("zapisywanie top score:\t\tOK\n");
}

void prepare_top_score_to_send(char buf[])			//przesyla top score do klienta
{
	if(DSP_MSG == 0)
	{
		printf("\nprepare_top_score_to_send\n");
	}
	struct top_score* ptr;
	int counter=1;				//pozycja od ktorej ma zapisywac
	int i=0;

	ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);
	if(ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej dl top score");
		exit(-1);
	}
	int j=0;
	for(i = 0 ; i < TS ; i++)
	{
		j=0;
		while(j < NICK_LENGTH)
		{
			buf[counter] = ptr->nick[j];
			j++;
			counter++;
		}
		buf[counter] = 9;
		counter++;

		j=0;
		while(j < POINTS_LENGTH)
		{
			buf[counter] = ptr->points[j];			//punkty do top scora w pamieci wspoldzielonej przechowywane sa jako char np 032 (3 komorki)
			j++;
			counter++;
		}
		buf[counter] = 10;
		counter++;

		ptr++;
	}
	shmdt(ptr);

	int tmp=0;
	for(tmp =0 ; tmp < BUF_SIZE ; tmp++)
	{
		if(DSP_MSG == 0)
		{
			printf("%d", buf[tmp]);
		}

	}

}

int count_points(char board[][BOARD_SIZE], int player_color)				//zwraca punkty gracza
{
	int i=0,j=0;
	int total_points;

	for(i = 0 ; i < BOARD_SIZE ; i++)
	{
		for(j = 0 ; j < BOARD_SIZE ; j++)
		{
			if(board[i][j] == player_color)
			{
				total_points++;
			}
		}
	}

	return total_points++;
}

int convert_char_to_int(char my_char[])         //zamienia POINTS_LENGTH - elementowego chara na inta
{

	int my_int=0;
	int i=0;
	int mult=1;

	for(i=POINTS_LENGTH ; i > 0 ; i--)
	{
		my_int = my_int + my_char[i-1]*mult;
		mult=mult*10;
	}

	if(DSP_MSG == 0)
	{
		printf("\nconvert_char_to_int char %c %c %c int %d\n", my_char[0], my_char[1],my_char[2],my_int);
	}
	return my_int;
}

void convert_int_to_char(char my_char[], int* my_int)		//zmienia my_int na POINTS_LENGTH - elementowego chara i wynika zapisuje do my_char
{
	int i=POINTS_LENGTH-1;
	int j=0;

	for(j = 0 ; j < POINTS_LENGTH ; j++)
	{
		my_char[j] = 0;
	}

	while((my_int > 0)&&(i >= 0))
	{
		my_char[i] = *my_int % 10;
		*my_int = *my_int / 10;
		i--;
	}

	if(DSP_MSG == 0)
	{
		printf("\nconvert_int_to_char char: '%c' '%c' '%c' int '%d'\n", my_char[0], my_char[1],my_char[2], *my_int);
	}
}

void get_login_from_message(char buf[], char my_login[])	//wyciaga login z wiadomosci wynik w my_login
{
	int i=0;

	for(i = 0 ; i < NICK_LENGTH ; i++)
	{
		my_login[i] = 0;
	}
	for(i = 0 ; i < NICK_LENGTH ; i++)
	{
		my_login[i] = buf[i+1];
	}

	if(DSP_MSG == 0)
	{
		printf("g\net_login_from_message %c %c %c %c %c %c %c %c \n", my_login[0], my_login[1],my_login[2],my_login[3],my_login[4],my_login[5],my_login[6],my_login[7]);
	}
}


	/**
	 * <p>Funkcja sprawdza:
	 * 1. Mozliwosc wykonania ruchu dla podanej planszy array i ruchu jako wspolzednych x y
	 * 2. Ocenia jakosc ruchu.
	 * 3. Wykonuje ruch.</p>
	 *
	 * @param array Tablica z aktualnym stanem planszy.
	 * @param x Wspolzedna x planszy na ktora ma zostac postawiony pionek
	 * (jezeli podamy -1 zostanie wygenerowana plansza poczatkowa)
	 * @param y Wspolzedna y planszy na ktora ma zostac postawiony pionek
	 * (jezeli podamy -1 zostanie wygenerowana plansza poczatkowa)
	 * @param color Kolor gracza dla ktorego wykonywany jest ruch WAZNE!
	 * @param func okresla dzialanie funkcji: 0 - funkcja bedzie sprawdzac mozliwosc wykonania
	 * danego ruchu i oceni jego skutecznosc, 1 - funkcja wykona dany ruch.
	 * Najpierw wywolujemy funkcje z parametrem func = 0 i sprawdzamy czy
	 * zwraca ona wartosc > 0 jesli tak to wywolujemy funkcje z parametrem
	 * func = 1 zeby wykonac ruch.
	 * @return Wartosc zwracana wazna jest tylko wtedy gdy wywolujemy funkcje
	 * z parametrem func = 0. Zwrocona liczba oznacza skutecznosc ruchu - liczba
	 * pionkow przeciwnika ktora zostanie zbita przez ten ruch. Jezeli funkcja
	 * zwroci 0 oznacza to ze ruch jest NIEDOZWOLONY.
	 */
int game_processor(char array[][BOARD_SIZE], char x, char y, char color, char func)
{
	int i=0, j=0, k=0;
	int rozmiar_planszy = 10;
	char oblicz_wartosc = 0;
	char modyfikuj_tabele = 1;
	char puste_pole = 0;

	if ((x== -1)||(y== -1))
	{
		for (i=0; i<rozmiar_planszy; i++)
		{
			for (j=0; j<rozmiar_planszy; j++)
			{
				if ((i==4)&&(j==4))
				{
					array[i][j]=1;
				}
				else if ((i==4)&&(j==5)) {
					array[i][j]=2;
				} else if ((i==5)&&(j==4)) {
					array[i][j]=2;
				} else if ((i==5)&&(j==5)) {
					array[i][j]=1;
				} else {
					array[i][j] = puste_pole;
				}
			}
		}
		return 0;
	}

	if (array[(int)x][(int)y]== 0)
	{
		int value = 0;
		for (i=1;(x-i>=0);i++) 			//po kolumnach w lewo
		{
			if ((i==1)&&(array[(int)(x-i)][(int)(y)]!=((color%2)+1))) {
				break;
			} else if (array[(int)(x-i)][(int)(y)]==puste_pole) {
				break;
			} else if (array[(int)(x-i)][(int)y]==((color%2)+1)) {
				continue;
			} else if (array[(int)(x-i)][(int)y]==color) {
				if (func==oblicz_wartosc) {
					value = value + i -1;
					break;
				} else if (func==modyfikuj_tabele) {
					for (k=x;k>=x-i;k--) {
						array[(int)k][(int)y]=color;
					}
					break;
				}
			}
		}
		for (i=1;(x+i<rozmiar_planszy);i++) 		//po kolumnach w prawo
		{
			if ((i==1)&&(array[(int)(x+i)][(int)y]!=((color%2)+1))) {
				break;
			} else if (array[(int)(x+i)][(int)y]==puste_pole) {
				break;
			} else if (array[(int)(x+i)][(int)y]==((color%2)+1)) {
				continue;
			} else if (array[(int)(x+i)][(int)y]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for (k=x;k<=x+i;k++) {
						array[(int)k][(int)y]=color;
					}
					break;
				}
			}
		}
		for (i=1;(y-i>=0);i++) 		//po wierszach w gore
		{
			if (((int)i==1)&&(array[(int)x][(int)(y-i)]!=((color%2)+1))) {
				break;
			} else if (array[(int)x][(int)(y-i)]==puste_pole) {
				break;
			} else if (array[(int)x][(int)(y-i)]==((color%2)+1)) {
				continue;
			} else if (array[(int)x][(int)(y-i)]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for (k=y;k>=y-i;k--) {
						array[(int)x][(int)k]=color;
					}
					break;
				}
			}
		}
		for (i=1;(y+i<rozmiar_planszy);i++) 		//po wierszach w dol
		{
			if ((i==1)&&(array[(int)x][(int)(y+i)]!=((color%2)+1))) {
				break;
			} else if (array[(int)x][(int)(y+i)]==puste_pole) {
				break;
			} else if (array[(int)x][(int)(y+i)]==((color%2)+1)) {
				continue;
			} else if (array[(int)x][(int)(y+i)]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for (k=y;k<=y+i;k++) {
						array[(int)x][(int)k]=color;
					}
					break;
				}
			}
		}
		for (i=1;((x+i<rozmiar_planszy)&&(y+i<rozmiar_planszy));i++) { //kolumny prawo wiersze dol
			if ((i==1)&&(array[(int)(x+i)][(int)(y+i)]!=((color%2)+1))) {
				break;
			} else if (array[(int)(x+i)][(int)(y+i)]==puste_pole) {
				break;
			} else if (array[(int)(x+i)][(int)(y+i)]==((color%2)+1)) {
				continue;
			} else if (array[(int)(x+i)][(int)(y+i)]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for (k=x,j=y;k<=x+i&&j<=y+i;k++,j++) {
						array[(int)k][(int)j]=color;
					}
					break;
				}
			}
		}
		for (i=1;((x-i>=0)&&(y-i>=0));i++) { //kolumny lewo wiersze gora
			if ((i==1)&&(array[(int)(x-i)][(int)(y-i)]!=((color%2)+1))) {
				break;
			} else if (array[(int)(x-i)][(int)(y-i)]==puste_pole) {
				break;
			} else if (array[(int)(x-i)][(int)(y-i)]==((color%2)+1)) {
				continue;
			} else if (array[(int)(x-i)][(int)(y-i)]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for ( k=x,j=y;k>=x-i&&j>=y-i;k--,j--) {
						array[(int)k][(int)j]=color;
					}
					break;
				}
			}
		}
		for ( i=1;((x-i>=0)&&(y+i<rozmiar_planszy));i++) { //kolumny lewo wiersze dol
			if ((i==1)&&(array[(int)(x-i)][(int)(y+i)]!=((color%2)+1))) {
				break;
			} else if (array[(int)(x-i)][(int)(y+i)]==puste_pole) {
				break;
			} else if (array[(int)(x-i)][(int)(y+i)]==((color%2)+1)) {
				continue;
			} else if (array[(int)(x-i)][(int)(y+i)]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for ( k=x,j=y;k>=x-i&&j<=y+i;k--,j++) {
						array[(int)k][(int)j]=color;
					}
					break;
				}
			}
		}
		for ( i=1;((x+i<rozmiar_planszy)&&(y-i>=0));i++) { //kolumny prawo wiersze gora
			if ((i==1)&&(array[x+i][y-i]!=((color%2)+1))) {
				break;
			} else if (array[x+i][y-i]==puste_pole) {
				break;
			} else if (array[x+i][y-i]==((color%2)+1)) {
				continue;
			} else if (array[x+i][y-i]==color) {
				if (func==oblicz_wartosc) {
					value = value + i-1;
					break;
				} else if (func==modyfikuj_tabele) {
					for ( k=x,j=y;k<=x+i&&j>=y-i;k++,j--) {
						array[k][j]=color;
					}
					break;
				}
			}
		}
		return value;
	}
	else {
		return 0;
	}
}


	/**
	 * <p>Funkcja generujaca ruch komputera. Szuka najlepszego ruchu - zbijajacego
	 * najwiecej pionkow dla danej planszy.</p>
	 * @param array Referencja do tablicy z aktualna plansza (ZOSTANIE ZMIENIONA)
	 * @param color Kolor jaki ma komputer
	 * @return 0 - jezeli komputer nie moze wykonac ruchu, 1 - ruch zostal wykonany, nastapila zmiana tablicy array
	 */
int cpuAI(char array[][BOARD_SIZE], char color)
{
	int rozmiar_planszy = 10;
	int puste_pole = 0;
	int best_move = 0;
	char movx = -1;
	char movy = -1;
	int temp=0;
	char i=0, j=0;

	for (i=0;i<rozmiar_planszy;i++) {
		for (j=0;j<rozmiar_planszy;j++) {
			if (array[(int)i][(int)j]==puste_pole) {
				temp=game_processor(array, i, j, color, (char)0); //sprawdzam jak dobry jest ten ruch


				if (temp>best_move) { //zapamietuje najlepszy ruch
					best_move = temp;
					movx = i;
					movy = j;
				}
			}
		}
	}
	if ((movx==-1)||(movy==-1)) { //jezeli nie zostal znaleziony zaden ruch zwroc 0
		return 0;
	} else {
		game_processor(array, movx, movy, color, (char)1); //jezeli zostal znaleziony najlepszy ruch to go wykonaj
		return 1;
	}
}

int isBlocked(char tab[][BOARD_SIZE], char color)
{
	int rozmiar_planszy = 10;
	int puste_pole = 0;
	char i=0, j=0;

	for (i=0;i<rozmiar_planszy;i++) {
		for (j=0;j<rozmiar_planszy;j++) {
			if (tab[(int)i][(int)j]==puste_pole) {
				if (game_processor(tab, i, j, color, (char)0)>0) {
					return 0;
				}
			}
		}
	}
	return 1;
}





int add_result_to_top_score(char my_points[], char my_nick[])      //jezeli my_points sa lepsze niz ktorys wpis w top_score zapisuje go i zwraca 0; w przypadku gdy my_points maja zbyt mala wartosc zwraca -1
{
	if(DSP_MSG == 0)
	{
		printf("add-result-to-top-score: points: '%d', '%d' '%d' nick: '%c' '%c' '%c' '%c' '%c' '%c' '%c' '%c' '%c' '%c'", my_points[0],my_points[1],my_points[1], my_nick[0],my_nick[1],my_nick[2],my_nick[3],my_nick[4],my_nick[5],my_nick[6],my_nick[7],my_nick[8],my_nick[9]);
	}
	//UWAGA PAMIETAC O ODPOWIEDNIM SFORMATOWANIU DANYCH DO TEJ F-CJI my_points tablica char o wielkosci POINTS_LENGTH (wartosci od 0 do 9) nie ascii; my_nick tablica char o dlugosci NICK_LENGTH wypelniona nickiem a wolne pola 0 , nie '0'
	int i=0,j=0,k=0;
	char cur_nick[NICK_LENGTH];
	char cur_points[POINTS_LENGTH];
	char suc_nick[NICK_LENGTH];
	char suc_points[POINTS_LENGTH];

	j=0;
	while(j < NICK_LENGTH)
	{
		cur_nick[j] = 0;
		j++;
	}
	j=0;
	while(j < POINTS_LENGTH)
	{
		cur_points[j] = 0;
		j++;
	}
	j=0;
	while(j < NICK_LENGTH)
	{
		suc_nick[j] = 0;
		j++;
	}
	j=0;
	while(j < POINTS_LENGTH)
	{
		suc_points[j] = 0;
		j++;
	}
	j=0;

	char tab_from_shm[POINTS_LENGTH];	//w tablicy beda przechowywane liczby z pamieci wspoldzielonej pomniejszone o 48
	for(j=0; j < POINTS_LENGTH; j++)	//najpierw zerujemy tablice
	{
		tab_from_shm[j] = 0;
	}

	struct top_score* ptr; //wskaznik do pamieci wspoldzielonej gdzie ma byc zapisany top score

        ptr = (struct top_score*)shmat(shmid_tsc, NULL, 0);    //przylaczenie pamieci wspoldzielonej
        if(ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w add_result_to_top_score");
		exit(-1);
	}

        int my_points_int=0;
        int points_int=0;
        for(i = 0 ; i < TS ; i++)
        {
        	for(j=0; j < POINTS_LENGTH; j++)
        	{
        		tab_from_shm[j] = ptr->points[j] - 48;
        	}

        	points_int = convert_char_to_int(tab_from_shm);
        	my_points_int = convert_char_to_int(my_points);
        	if(points_int < my_points_int)
        	{

			j=0;
			while(j < NICK_LENGTH)			//zapamietujemy w cur_nick rekord w miejsce ktorego ma byc wstawiona nowa wartosc
			{
				cur_nick[j] = ptr->nick[j];
				j++;
			}

			j=0;
			while(j < POINTS_LENGTH)
			{
				cur_points[j] = ptr->points[j];
				j++;
			}


			j=0;
			while(j < NICK_LENGTH)			//wstawiamy nowa wartosc do rekordu (stara wartosc mamy zapamietana w cur_nick i cur_points
			{
				ptr->nick[j] = my_nick[j];
				j++;
			}
//
j=0;
while(j < POINTS_LENGTH)
{
	ptr->points[j] = 0;
	j++;
}
//
			j=0;
			while(j < POINTS_LENGTH)
			{
				ptr->points[j] = my_points[j]+48;//uwaga tu musimy zrobic +48 bo w pliku char przechowujemy jako ascii
				j++;
			}

			ptr++;						//przesuwamy sie na kolejny rekord
        		for(k=i; k < TS; k++)				//jezeli nie jest koniec tablicy przechodzimy po kolejnych rekordach
        		{
        			j=0;
				while(j < NICK_LENGTH)			//kopiujemy rekord na ktory aktualnie pokazujemy
				{
					suc_nick[j] = ptr->nick[j];
					j++;
				}
//
j=0;
while(j < POINTS_LENGTH)
{
	suc_points[j] = 0;
	j++;
}
//
				j=0;
				while(j < POINTS_LENGTH)
				{
					suc_points[j] = ptr->points[j];
					j++;
				}

//
j=0;
while(j < NICK_LENGTH)			//wstawiamy do pamieci wspoldzielonej POPRZEDNI rekord
{
	ptr->nick[j] = 0;
	j++;
}
j=0;
while(j < POINTS_LENGTH)
{
	ptr->points[j] = 0;
	j++;
}
//
        			j=0;
				while(j < NICK_LENGTH)			//wstawiamy do pamieci wspoldzielonej POPRZEDNI rekord
				{
					ptr->nick[j] = cur_nick[j];
					j++;
				}
				j=0;
				while(j < POINTS_LENGTH)
				{
					ptr->points[j] = cur_points[j] ;
					j++;
				}

				j=0;
				while(j < NICK_LENGTH)			//POPRZEDNI rekord nie jest juz potrzebny wiec w jego miejsce wstawiamy NASTEPNY
				{
					cur_nick[j] = suc_nick[j];
					j++;
				}
//
j=0;
while(j < POINTS_LENGTH)
{
	cur_points[j] = 0;
	j++;
}
//
				j=0;
				while(j < POINTS_LENGTH)
				{
					cur_points[j] = suc_points[j] ;
					j++;
				}


				ptr++;
        		}

        		break;		//wychodzimy z nadrzednego fora
        	}

                ptr++;
        }
        shmdt(ptr);     //odlaczamy pamiec wspoldzielona

        return -1;
}



int add_soc_dsc(int dsc)			//dodaje deskryptor gniazdka do pamieci wspoldzielonej; zwraca 0 - sukces; -1 blad(nie ma juz miejsca w pamieci wspoldzielonej)
{
	if(DSP_MSG == 0)
	{
		printf("dodawany desk %d\n", dsc);
	}
	int* shm_addr;
	int i=0;

	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		if(shm_addr[i] == 0)
		{
			semop(semid_usr, &Psoc, 1);		//opuszczanie semafora
			shm_addr[i] = dsc;
			semop(semid_usr, &Vsoc, 1);		//podnoszenie semafora
			shmdt(shm_addr);
			return 0;
		}
	}
	shmdt(shm_addr);
	return -1;
}

int remove_soc_dsc(int dsc)			//usuwa deskryptor gniazdka z pamieci wspoldzielonej; zwraca 0 sukces; -1 blad(nie ma takiego gniazda w pamieci wspoldzielonej)
{
	int* shm_addr;
	int i=0;

	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		if(shm_addr[i] == dsc)
		{
			semop(semid_usr, &Psoc, 1);		//opuszczanie semafora
			shm_addr[i] = 0;
			semop(semid_usr, &Vsoc, 1);		//podnoszenie semafora

			shmdt(shm_addr);
			return 0;
		}
	}
	shmdt(shm_addr);
	return -1;
}

void close_dsc(int dsc_to_close)		//zamyka deskryptor podany jako parametr
{

	int* shm_addr;
	int i=0;

	semop(semid_usr, &Psoc, 1);		//opuszczanie semafora
	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		if(shm_addr[i] != 0)
		{
			if(shm_addr[i] == dsc_to_close)
			{
				close(shm_addr[i]);
				if(DSP_MSG == 0)
				{
					printf("zamknieto: %d gniazdko\n",shm_addr[i]);
				}
			}
		}
	}
	shmdt(shm_addr);
	semop(semid_usr, &Vsoc, 1);		//podnoszenie semafora
}

void close_all_soc_dsc()
{

	int* shm_addr;
	int i=0;

	semop(semid_usr, &Psoc, 1);		//opuszczanie semafora
	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		if(shm_addr[i] != 0)
		{
			close(shm_addr[i]);
			if(DSP_MSG == 0)
			{
				printf("zamknieto: %d gniazdko\n",shm_addr[i]);
			}
		}
	}
	shmdt(shm_addr);
	semop(semid_usr, &Vsoc, 1);		//podnoszenie semafora
}
void close_soc_dsc_admin(int dsc, int dsc_server)		//zamyka wszysktkie deskryptory oprocz tego podanego jako argument
{

	int* shm_addr;
	int i=0;

	semop(semid_usr, &Psoc, 1);		//opuszczanie semafora
	shm_addr = shmat(shmid_soc, NULL, 0);
	for(i = 0 ; i < (SOC_NMB) ; i++)
	{
		if(shm_addr[i] != 0)
		{
			if((shm_addr[i] != dsc)&&(shm_addr[i])!= dsc_server)
			{
				close(shm_addr[i]);
				if(DSP_MSG == 0)
				{
					printf("zamknieto w admin: %d gniazdko\n",shm_addr[i]);
				}
			}
		}
	}
	shmdt(shm_addr);
	semop(semid_usr, &Vsoc, 1);		//podnoszenie semafora
}

void clean_buf(char buf[BUF_SIZE])
{
	int i=0;

	for(i = 0 ; i < BUF_SIZE ; i++)
	{
		buf[i] = 0;
	}

}

void prepare_board(char board[][BOARD_SIZE])		//przygotowuje plansze do gry - wypelnia zerami (plansza typu char)
{
	int i,j;

	for(i = 0 ; i < BOARD_SIZE ; i++)
	{
		for(j = 0; j < BOARD_SIZE ; j++)
		{
			board[i][j] = 0;
			//printf("%d", board[i][j]);
		}
		//printf("\n");
	}
}

void convert_board_to_send(char board[][BOARD_SIZE], char dst[])	//przeksztalca tablice int 2 wymiarowa na 1 wymiarowa tablice char (dst od indeksu 1 bo indeks 0 zarezerwowany na naglowek)
{
	if(DSP_MSG == 0)
	{
		printf("\nconvert board to send\n");
	}
	int start=1;		//mowi od ktorej pozycji ma zapisywac w tablicy dst
	int i=0, j=0;


	for(i = 0 ; i < BOARD_SIZE ; i++)
	{
		for(j = 0 ; j < BOARD_SIZE ; j++)
		{
			dst[start] = board[j][i];
			if(DSP_MSG == 0)
			{
				printf("%d ", dst[start]);
			}

			start++;
		}
		if(DSP_MSG == 0)
		{
			printf("\n");
		}
	}
	/*


	int xi, xj, start = 1;		//start - mowi od ktorej pozycji ma byc zapisywana tablica dst
	char copy = '1';
	int xtmp;

	for(xi = 0 ; xi < BOARD_SIZE ; xi++)
	{
		for(xj = 0 ; xj < BOARD_SIZE ; xj++)
		{
			printf("%d", board[xi][xj]);

			xtmp = board[xi][xj];
			int c = sprintf(&copy,"%d",xtmp);
			printf("[%d]", c);
			  	//sprintf(&j,"%d",i);
			dst[start] = copy;
			start++;
		}
		printf("#\n");

	}
	printf("==============\n");
*/

}



int check_number_of_users()		//sprawdza czy nie zostala przekroczona maksymalna liczba uzytkownikow na serwerze; jezeli tak zwraca -1; jezeli nie 0
{
	int* shm_addr;
	int nmb_users;

	shm_addr = shmat(shmid_usr, NULL, 0);
	if(shm_addr == NULL)
	{
		perror("Blad podczas przylaczania pamieci podrecznej w check_number_of_users");
		exit(-1);
	}

	nmb_users = shm_addr[0];               //tylko czytamy wiec nie trzeba zakladac semafora

	shmdt(shm_addr);

	if(DSP_MSG == 0)
	{
		printf("check_number_of_users%d\n", nmb_users);
	}

	if(nmb_users >= MAX_USR)	//jezeli w systemie jest juz zalogowana maksymalna liczba uzytkownikow tez zostaje zwrocone -1
		return -1;
	else
		return 0;
}

void increment_number_of_users()	//uruchamiana w watku; zwieksza liczbe uzytkownikow w systemie i dodaje pid watku do tablicy z pidami
{
	if(DSP_MSG == 0)
	{
		printf("increment_number_of_users\n");
	}

	int* shmaddr_usr;
	int* shmaddr_pid;
	int i=0;

	shmaddr_usr = shmat(shmid_usr, NULL, 0);
	if(shmaddr_usr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w increment_number_of_users dla liczby uzytkownikow");
		exit(-1);
	}


	semop(semid_usr, &P, 1);		//opuszczanie semafor
	//printf("opuscil semafor\n");
	shmaddr_usr[0]++;

	semop(semid_usr, &V, 1);		//podnoszenie samafora
	//printf("podniosl semafor\n");
	shmdt(shmaddr_usr);


	shmaddr_pid = shmat(shmid_pid, NULL,0);
	if(shmaddr_pid == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w increment_number_of_users dla pidow");
		exit(-1);
	}

	for(i = 0 ; i < MAX_USR ; i++)
	{
		if(shmaddr_pid[i] == 0)
		{
			if(DSP_MSG == 0)
			{
				printf("pid: %d\n", shmaddr_pid[i]);
			}
			semop(semid_usr, &Ppid, 1);
			shmaddr_pid[i] = getpid();
			semop(semid_usr, &Vpid, 1);
			if(DSP_MSG == 0)
			{
				printf("pid: %d\n", shmaddr_pid[i]);
			}
			break;
		}
	}

	shmdt(shmaddr_pid);

}

void decrement_number_of_users()        //zmniejsza liczbe uzytkownikow (shmaddr_usr--) i usuwa pid z tablicy z pidami (shmid_pid)
{
	if(DSP_MSG == 0)
	{
		printf("decrement_number_of_users\n");
	}

	int* shmaddr_usr;
	int* shmaddr_pid;
	int i=0;

	shmaddr_usr = shmat(shmid_usr, NULL, 0);
	if(shmaddr_usr == NULL)
	{
		perror("Blad podczas przylaczania pamieci podrecznej w decrement_number_of_users");
		exit(-1);
	}

	semop(semid_usr, &P, 1);

	shmaddr_usr[0]--;

	semop(semid_usr, &V, 1);

	shmdt(shmaddr_usr);


	shmaddr_pid = shmat(shmid_pid, NULL,0);
	if(shmaddr_pid == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w increment_number_of_users dla pidow");
		exit(-1);
	}
	for(i = 0 ; i < MAX_USR ; i++)
	{
		if(shmaddr_pid[i] == getpid())
		{
			if(DSP_MSG == 0)
			{
				printf("pid: %d\n", shmaddr_pid[i]);
			}
			semop(semid_usr, &Ppid, 1);
			shmaddr_pid[i] = 0;
			semop(semid_usr, &Vpid, 1);
			if(DSP_MSG == 0)
			{
				printf("pid: %d\n", shmaddr_pid[i]);
			}
			break;
		}
	}

	shmdt(shmaddr_pid);

}



int save_mlp_id(int pid)		//zapisuje do pamieci wspoldzielonej pid watku gracza, ktory chce grac w trybie multiplayer; zwraca -1 w przypadku bledu (zbyt duzo uzytkownikow); 0 gdy gracz bedzie glownym graczem(master); 1 gdy gracz bedzie sie dolaczal(slave)
{
	int* shm_addr;
	int i=0;

	shm_addr = shmat(shmid_mlp, NULL, 0);

	for(i = 0 ; i < (2*MAX_USR) ; i++)	//najpierw gracz sprawdza czy moze sie przylaczyc do jakiejs rozgrywki
	{
		if( (i%2) != 0 )		//gracz chce sie dolaczyc do isteniejacej rozgrywki wiec sprawdza nieparzyste pola
		{
			if(shm_addr[i] == 0)	//nastepnie sprawdza czy pole nie jest juz zajete przez kogos innego
			{
				if(shm_addr[i-1] != 0)	//sprawdza czy jest gracz ktory stworzyl rozgrywke
				{
					semop(semid_usr, &Pmlp, 1);
					shm_addr[i] = pid;	//jezeli wszystkie warunki sa spelnione gracz wpisuje pid swojego watku
					semop(semid_usr, &Vmlp, 1);
					shmdt(shm_addr);
					return 1;
				}
			}
		}
	}

	for(i = 0 ; i < (2*MAX_USR) ; i++)		//dojscie do tej petli oznacza ze gracz bedzie zakladal rozgrywke multiplayer (nie mogl sie dolaczyc do istniejacej rozgrywki)
	{
		if( (i%2) == 0 )		//najpierw gracz wybiera odpowiednie pole (parzyste)
		{
			if(shm_addr[i] == 0)	//sprawdzamy czy pole nie jest zajete przez innego gracza
			{
				semop(semid_usr, &Pmlp, 1);
				shm_addr[i] = pid;	//jezeli wszystkie warunki sa spelnione gracz wpisuje pid swojego watku
				semop(semid_usr, &Vmlp, 1);
				shmdt(shm_addr);
 				return 0;
			}
		}
	}

	shmdt(shm_addr);
	return -1;		//oznacza to ze w systemie znalazlo sie zbyt duzo uzytkownikow i nie udalo sie wpisac pidu do pamieci
}

int remove_mlp_id(int pid)	//po skonczonej rozgrywce wieloosobowej gracze usuwaja swoje pidy z tablicy; zwraca 0 gdy pid zostal znaleziony; -1 w przypadku bledu - nie udalo sie odnalez pid
{
		int i=0;
		int* shm_addr;
		shm_addr = shmat(shmid_mlp, NULL, 0);

		for(i = 0 ; i < (2*MAX_USR) ; i++)
		{
			if(shm_addr[i] == pid)
			{
				semop(semid_usr, &Pmlp, 1);
				shm_addr[i] = 0;
				semop(semid_usr, &Vmlp, 1);
				shmdt(shm_addr);
				return 0;
			}

		}

		return -1;
}

int login(int cdesk, char* buf, int desk)		//odpowiada za zainicjalizowanie gry; zwraca 0 w przypadku prawidlowego zalogowania; -1 w przypadku wystapienia bledu
{
	int ret_val;
	char send[BUF_SIZE];

	if(buf[0] == WM_HEAD)	//obsluga panelu administratora
	{
		if(DSP_MSG == 0)
		{
			printf("\nnr soc %d ", cdesk);
		}


		if(buf[1] == WM_HEAD)			//sprawdzanie statusu przez panel administratora
		{
			send[0] = WM_HEAD;
			//write(cdesk,send, sizeof(send));
		}
		else if(buf[1] == WM_NMBUSR)		//aktualna liczba uzytkownikow
		{
			int users=0;
			int* shm_addr;
			shm_addr = shmat(shmid_usr, NULL, 0);
			if(shm_addr == NULL)
			{
				perror("Blad podczas przylaczania pamieci podrecznej w login");
				exit(-1);
			}

			users = shm_addr[0];               //tylko czytamy wiec nie trzeba zakladac semafora

			shmdt(shm_addr);

			send[1] = users;

			send[0] = WM_NMBUSR;
			write(cdesk,send, sizeof(send));

		}
		else if(buf[1] == WM_RESTART)		//restartowanie serwera
		{

			save_top_score();
			close_soc_dsc_admin(cdesk, desk);
			kill_all_threads();
			remove_queue();
			remove_shared_memory();
			remove_semafor();

			create_queue();
			create_shared_memory();
			create_semafor();
			get_top_score();

			//send[0] = WM_RESTART;
			//write(cdesk,send, sizeof(send));
		}
		else if(buf[1] == WM_END)		//konczenie pracy serwera
		{
			save_top_score();
			close_all_soc_dsc();
			kill_all_threads();
			remove_queue();
			remove_shared_memory();
			remove_semafor();

			exit(0);
		}
		else if(buf[1] == WM_MAXUSR)		//maksymalna liczba uzytkownikow w systemie
		{
			send[1] = MAX_USR;
			send[0] = WM_MAXUSR;
			write(cdesk,send, sizeof(send));
		}


		return 2;
	}

	clean_buf(send);

	//printf("login zaw buf: %c %c %c\n", buf[0], buf[1], buf[2]);

	ret_val = check_number_of_users();
	if(ret_val == 0)		//jezeli liczba uzytkownikow w systemie nie jest zbyt duza
	{
		send[0] = LOG_HEAD;
		write(cdesk,send, sizeof(send));	//wyslana zostaje wiadomosc o potwierdzeniu logowania ##USTAWIC ODPPOWIEDNIA DLUGOSC sizeof(send)
		return 0;
	}
	else		//jezeli blad w logowaniu - przekroczona maksymalna liczba uzytkownikow w systemie
	{
		send[0] = ERR_HEAD;
		write(cdesk,send, sizeof(send));	//wysyla wiadomosc o bledzie podczas logowania ##USTAWIC TRESC
		close(cdesk);				//zamykamy gniazdo od naszego accepta
		return -1;
	}

}



void singleplayer(int cdesk)				//f-cja dzialajaca w watku; gra z komputerem
{
	if(DSP_MSG == 0)
	{
		printf("singleplayer\n");
	}
	int my_points_int=0;						//ilosc punktow zdobytych przez gracza
	int ret_val=0;
	int flag = 0;						//mowi czy jest to ostatnia plansza
	char my_points_char[POINTS_LENGTH];				//ilosc punktow zdobytych przez gracza w POINTS_LENGTH elementowym char
	char my_login[NICK_LENGTH];


	char buf[BUF_SIZE];
	char board[BOARD_SIZE][BOARD_SIZE];			//przechowuje aktualny stan gry

	clean_buf(buf);
	prepare_top_score_to_send(buf);					//najpierw wysylamy topscore do klienta
	buf[0] = TS_HEAD;
	write(cdesk, buf,sizeof(buf));

	clean_buf(buf);
	prepare_board(board);

	game_processor(board,(char)-1,(char)-1,(char)1,(char)0);	//przygotowanie planszy

	while(1)
	{

		if(DSP_MSG == 0)
		{
			printf("\n.\n");
		}
		convert_board_to_send(board, buf);
		buf[0] = GAME_HEAD;
		write(cdesk, buf,sizeof(buf));		//wysyla plansze do klienta
		if(flag == 1)				//oznacza to ze byla to ostatnia plansza
		{					//sprawdza kto wygral i wysyla top score do klienta
			if(DSP_MSG == 0)
			{
				printf("\nwszedl do flagi 1\n");
			}
			clean_buf(buf);
			buf[0] = END_HEAD;
			my_points_int = who_is_winner(board, 1);		//zwraca ilosc punktow zdobyta przez gracza 1
			int pc_points_int = who_is_winner(board, 2);
			if(my_points_int > pc_points_int)
			{
				buf[1] = 1;			//wygrana klienta
			}
			else
			{
				buf[1] = 0;			//wygrana pc
			}

			write(cdesk, buf,sizeof(buf));		//wysyla informacje o tym kto wygral do klienta

		}

		clean_buf(buf);
		read(cdesk, buf, sizeof(buf));		//odbiera odpowiedz od klienta

		//buf[1]	//wspolrzedna x
		//buf[2]	//wspolrzedna y


		if(buf[0] == END_HEAD)				//klient przyslal login
		{
			//printf("%c,%c,%c,%c", buf[0],buf[1],buf[2],buf[3]);
			get_login_from_message(buf, my_login);	//wyciaga login  z wiadomosci
			convert_int_to_char(my_points_char, &my_points_int);	//konwertuje punkty w int na punkty w char (zeby dobrze zapisac w pliku)

			int temp=0;	//usunac
			temp = add_result_to_top_score(my_points_char, my_login);
			if(DSP_MSG == 0)
			{
				printf("\nto zwraca add result to top score : %d\n", temp);
			}
			clean_buf(buf);
			prepare_top_score_to_send(buf);
			buf[0] = TS_HEAD;
			write(cdesk, buf,sizeof(buf));		//wysylany jest top score to klienta

			if(DSP_MSG == 0)
			{
				printf("koniec single playera\n");
			}
			break;					//zakancza rozgrywke (watek zostanie zakonczony)
		}
		if(buf[0] == ERR_HEAD)				//wystapil jakis blad
		{

			break;					//zakancza rozgrywke (watek zostanie zakonczony)
		}

		ret_val = game_processor(board, buf[1], buf[2], (char)1, (char)0);		//wykonany zostaje ruch gracza
		if(ret_val > 0)									//jezeli jest dozwolony to ruch wykonuje komp
		{
			game_processor(board, buf[1], buf[2], (char)1, (char)1);
			do
			{
				if(DSP_MSG == 0)
				{
					printf("\n,\n");
				}
				cpuAI(board, (char)2);
			}
			while((isBlocked(board, (char)1))&&(!isBlocked(board, (char)2)));

			if ((isBlocked(board, (char)1))&&(isBlocked(board, (char)2)))		//koniec gry gracze zablokowani
			{
				if(DSP_MSG == 0)
				{
					printf("\nwszedl do block\n");
				}
				flag = 1;
				buf[1023] = 0;

			}

		}

	}

	if(DSP_MSG == 0)
	{
		printf("kkkoniec\n");
	}

	/*
*
			  //wygenerowanie planszy poczatkowej
			  gameProcessor(plansza,(char)-1,(char)-1,(char)1,(char)0);
			  //zamiana tablicy dwuwymiarowej na jednowymiarowa i dodanie odpowiedniego naglowka
			  out.write(connect((char)1,plansza));
		  } else if (temp[0]==1) {
			  int value;
			//sprawdzam czy ruch jest mozliwy do wykonania
			  value = gameProcessor(plansza, temp[1], temp[2], (char)1, (char)0);
			  System.out.println(value);
			  if (value>0) {
				  //jezeli tak to wykonaj ten ruch
				  gameProcessor(plansza, temp[1], temp[2], (char)1, (char)1);
				  do {
					  cpuAI(plansza, (char)2);
				  } while ((isBlocked(plansza, (char)1))&&(!isBlocked(plansza, (char)2)));
				  if ((isBlocked(plansza, (char)1))&&(isBlocked(plansza, (char)2))) {
					  //koniec gry gracze zablokowani

				  } else {
					  out.write(connect((char)1,plansza));
				  }


			  } else { //jezeli ruch byl zabroniony
				  //odsylam plansze zeby odblokowac gracza WAZNE !!
				  out.write(connect((char)1,plansza));
			  }
		  }
	      }

	*/


/*
int* shm_addr;
int gniazdko;
shm_addr = shmat(shmid_mlp, NULL, 0);
gniazdko = shm_addr[0];
shmdt(shm_addr);
printf("gniazdko %d\n", gniazdko);
*/

	/*
	//int tmp=0;
	char buf2[BUF_SIZE];
							//tmp=read(cdesk,buf,sizeof(buf));

	int cnt=0;
	while(cnt < 5)
	{
		write(cdesk, "elo\n",4);	//wysylamy wiadomosc
								//tmp=read(cdesk,buf2,sizeof(buf));
								//write(1, buf2, tmp);
		printf("while\n");
		cnt++;
	}
	printf("koniec watku\n");
	*/

}

int get_neighbour_pid(int my_pid)	//zwraca pid osoby z ktora gramy; -1 w przypadku bledu
{
	int i=0;
	int* shm_addr;
	int pid_val=0;
	shm_addr = shmat(shmid_mlp, NULL, 0);

	for(i = 0 ; i < (2*MAX_USR) ; i++)
	{
		if(shm_addr[i] == my_pid)
		{
			if( (i%2) == 0)		//oznacza to ze jest to uzytkownik zakladajacy gre
			{
				pid_val = shm_addr[i+1];	//bierzemy pid urzytkownika prawego (o nieparzystym numerze indeksu tablicy)
				shmdt(shm_addr);
				return pid_val;
			}
			else			//ocznacza uzytkownika dolaczajacego sie do gry
			{
				pid_val = shm_addr[i-1];	//bierzemy pid urzytkownika lewego (o parzystym numerze indeksu tablicy)
				shmdt(shm_addr);
				return pid_val;
			}

		}

	}
	return -1;
}

void create_mlp_game(int cdesk)		//gra multiplayer - gracz tworzacy rozgrywke (cdesk - deskryptor swojego gniazda)	##TU STWORZYC LOGIKE GRY I OBSLUGE CZATU
{
	if(DSP_MSG == 0)
	{
		printf("create_mlp_game\n");
	}

	char buf[BUF_SIZE];
	char board[BOARD_SIZE][BOARD_SIZE];
	int ret_val=0, flag =0, pid_ready=0;
	struct message msg_s, msg_r;				//struktura wiadomosci do wysylania i odbierania
	int neighbour_pid =0;	//jeszcze nie mozemy zabrac pida sasiada bo nie wiemy z kim gramypid osoby z ktora gramy
	if(neighbour_pid == -1)
	{
		buf[0] = ERR_HEAD;
		write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
		return;
	}
	//msg_s.type = neighbour_pid;				//ustawiamy typ wysylanej wiadomosci wiadomosci na pid sasiada (jeszcze nie mozemy tego zrobic poniewaz nie znamy sasiada)
	msg_r.type = getpid();					//odbieramy wiadomosci takiego typu jaki jest nasz pid
	/*
	convert_board_to_send(board, buf);
	write(cdesk, buf,sizeof(buf));		//wysyla plansze do klienta
	clean_buf(buf);
	read(cdesk, buf, sizeof(buf));		//odbiera odpowiedz od klienta
	*/


	prepare_board(board);					//czysci plansze

	game_processor(board,(char)-1,(char)-1,(char)1,(char)0);	//przygotowanie planszy
	convert_board_to_send(board,buf);

	//buf[0] = GAME_HEAD;
	//write(cdesk, buf,sizeof(buf));		//wysyla plansze do klienta master z poczatkowym stanem gry

	pid_ready = 0;
	while(1)
	{

		do
		{

			msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);

			if(msg_r.content[0] == ERR_TH)			//jezeli klient z join odlaczyl sie klient master wygrywa
			{
				//printf("join zostal rozlaczony\n");
				buf[0] = END_HEAD;
				buf[1] = 1;
				write(cdesk,buf, sizeof(buf));
				return;				//wychodzimy z f-cji aby poprawnie zakonczyl sie watek
			}


			if(pid_ready == 0)									//jezeli pierwsza komunikacja to watek master musi pobrac pid gracza z ktormy gra zeby mogl sie z nim komunikowac
			{
				neighbour_pid = get_neighbour_pid(getpid());
				msg_s.type = neighbour_pid;

				pid_ready=1;
			}


			if((ret_val = game_processor(board, msg_r.content[1], msg_r.content[2], (char)1, (char)0)) == 0)		//musimy odpytywac az slave wykona odpowiedni ruch
			{
				flag = 1;
			}
			else
			{

				game_processor(board, msg_r.content[1], msg_r.content[2], (char)1, (char)1);


				if ((isBlocked(board, (char)1))&&(isBlocked(board, (char)2)))
				{
					msg_s.content[0] = END_TH;
					buf[0] = END_HEAD;

					int master_point = who_is_winner(board, 2);
					int slave_point = who_is_winner(board, 1);

					if(master_point > slave_point)
					{
						msg_s.content[1] = 0;
						buf[1] = 1;
					}
					else
					{
						msg_s.content[1] = 1;
						buf[1] = 0;
					}

					write(cdesk,buf, sizeof(buf));				//wysylamy informacje o koncu rozgrywki do mastera
					ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy informacje o koncu rozgrywki do watku slave

					return;
				}

				if (isBlocked(board, (char)2))
				{
					if(DSP_MSG == 0)
					{
						printf("\nblokada\n");
					}
					flag = 1;
				}
				else
					flag = 0;

			}


			//board[(int)msg_r.content[1]][(int)msg_r.content[2]] = (char)1;


			convert_board_to_send(board,msg_s.content);
			msg_s.content[0] = GAME_TH;
			if(flag == 0)
				msg_s.content[1023] = 0;
			else
				msg_s.content[1023] = 1;
			msgsnd(msgq_id, &msg_s, msg_size,0);


			convert_board_to_send(board,buf);
			buf[0] = GAME_HEAD;
			if(flag == 0)
				buf[1023] = 1;
			else
				buf[1023] = 0;

			write(cdesk,buf, sizeof(buf));

		}
		while(flag == 1);




		do
		{
			read(cdesk, buf, sizeof(buf));

			if(buf[0] == ERR_HEAD)					//jezeli watek master wysyla blad przesylamy do joina ze wygral
			{
				msg_s.content[0] = ERR_TH;
				msgsnd(msgq_id, &msg_s, msg_size,0);
				return;
			}


			if((ret_val = game_processor(board, buf[1], buf[2], (char)2, (char)0)) == 0)		//musimy odpytywac az slave wykona odpowiedni ruch
			{
				flag = 1;
			}
			else
			{
				game_processor(board, buf[1], buf[2], (char)2, (char)1);


				if ((isBlocked(board, (char)1))&&(isBlocked(board, (char)2)))		//koniec gry gracze zablokowani
				{

					msg_s.content[0] = END_TH;
					buf[0] = END_HEAD;

					int master_point = who_is_winner(board, 2);
					int slave_point = who_is_winner(board, 1);

					if(master_point > slave_point)
					{
						msg_s.content[1] = 0;
						buf[1] = 1;
					}
					else
					{
						msg_s.content[1] = 1;
						buf[1] = 0;
					}

					write(cdesk,buf, sizeof(buf));				//wysylamy informacje o koncu rozgrywki do mastera
					msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy informacje o koncu rozgrywki do watku slave

					return;				//konczymy rozgrywke

				}


				if (isBlocked(board, (char)1))
				{
					flag = 1;
					if(DSP_MSG == 0)
					{
						printf("\nblokada\n");
					}
				}
				else
					flag = 0;

			}



			//board[(int)buf[1]][(int)buf[2]] = (char)2;

			convert_board_to_send(board, buf);
			buf[0] = GAME_HEAD;
			if(flag == 0)
				buf[1023] = 0;
			else
				buf[1023] = 1;
			write(cdesk,buf, sizeof(buf));


			convert_board_to_send(board, msg_s.content);

			msg_s.content[0] = GAME_TH;
			if(flag == 0)
				msg_s.content[1023] = 1;
			else
				msg_s.content[1023] = 0;

			msgsnd(msgq_id, &msg_s, msg_size,0);
		}
		while(flag == 1);






		/*
		printf("\n#czeka na odpowiedz od $\n");
		ret_val = msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);			//oczekuje na odpowiedz od watku slave
		if(ret_val == -1)
		{
			perror("blad podczas odbierania komunikatu");
			return;
		}
		if(flag == 0)									//jezeli pierwsza komunikacja to watek master musi pobrac pid gracza z ktormy gra zeby mogl sie z nim komunikowac
		{
			neighbour_pid = get_neighbour_pid(getpid());
			msg_s.type = neighbour_pid;
			printf("\n#master pidy %d %d\n", neighbour_pid, getpid());
		}
		flag=1;

		//analizuje co przyslal watek slave
		switch(msg_r.content[0])
		{
			case GAME_TH:					//otrzymalismy ruch od watku slave
			{

				//analizuje ruchy klienta slave (kolor 1)

				ret_val=0;
				while((ret_val = game_processor(board, msg_r.content[1], msg_r.content[2], (char)1, (char)0)) == 0)		//musimy odpytywac az slave wykona odpowiedni ruch
				{

					convert_board_to_send(board, msg_r.content);
					msg_s.content[0] = GAME_TH;				//wejscie do petli oznacza ze slave wykonal nieodpowiedni ruch
					msg_s.content[1023] = 1;				//zly ruch klient slave bedzie musial wykonac znowu ruch
					msgsnd(msgq_id, &msg_s, msg_size,0);			//wysylamy info do slave zeby powtorzyl ruch
					printf("\n#master czeka na slava bo ten mial zly ruch\n");
					msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);		//odczytujemy odpowiedz od slave

				}		//wyjscie z petli oznacza ze slave wykonal w koncu dobry ruch

				if(ret_val > 0)									//jezeli jest dozwolony to ruch wykonuje komp
				{
					game_processor(board, msg_r.content[1], msg_r.content[2], (char)1, (char)1);	//zatwierdzenie ruchu i zapisanie go na planszy

					//printf("zatwierdza ruchy slave");
					if ((isBlocked(board, (char)1))&&(isBlocked(board, (char)2)))		//koniec gry gracze zablokowani
					{
						printf("\nNIE\n");

						msg_s.content[0] = END_TH;
						buf[0] = END_HEAD;

						int master_point = who_is_winner(board, 2);
						int slave_point = who_is_winner(board, 1);

						if(master_point > slave_point)
						{
							msg_s.content[1] = 0;
							buf[1] = 1;
						}
						else
						{
							msg_s.content[1] = 1;
							buf[1] = 0;
						}

						write(cdesk,buf, sizeof(buf));				//wysylamy informacje o koncu rozgrywki do mastera
						ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy informacje o koncu rozgrywki do watku slave

						return;				//konczymy rozgrywke

					}


					convert_board_to_send(board,msg_s.content);
					msg_s.content[0] = GAME_TH;
					msg_s.content[1023] = 0;
					msgsnd(msgq_id, &msg_s, msg_size,0);	//wysylamy plansze do klienta slave zeby widzial swoj ruch (bedize on teraz czekal na ruch mastera)

					convert_board_to_send(board,buf);
					buf[0] = GAME_HEAD;
					buf[1023] = 1;
					write(cdesk,buf, sizeof(buf));//mozemy wysylac plansze do klienta master (zeby widzial ruch slave) i zeby wykonal ruch

				}



			break;
			}

			case CHAT_TH:						//otrzymano wiadomosc czat od watku slave
			{
				buf[0] = CHAT_HEAD;
				for(i = 1; i < BUF_SIZE ; i++)
				{
					buf[i] = msg_r.content[i];
				}
				write(cdesk,buf, sizeof(buf));			//wysylamy tresc wiadomosci czat do klienta master

			break;
			}

			case ERR_TH:						//otrzymano blad os watku slave
			{
				buf[0] = ERR_HEAD;				//wysylamy do klienta powiadomienie o bledzie

				write(cdesk,buf, sizeof(buf));
				return;						//w przypadku wystapienia bledu wychodzi

			break;
			}


			default:
			{
				printf("nieprawidlowy format wiadomosci od slave\n");
				return;
			break;
			}

		}	//koniec switcha analizujacego tresc przeslana przez watek salve



		printf("\n#r czeka na odp od mastera\n");
		clean_buf(buf);
		read(cdesk, buf, sizeof(buf));		//czeka na ruch klienta master

		//analiza tego co przysyla klient master i odpowiednia reakcja
		switch(buf[0])
		{
			case GAME_HEAD:				//klient master wykonal ruch i wysylana jest informacja do sasiedniego watku slave z cala plansza
			{

				printf("\n#analizuje ruch mastera\n");
				//analizuje ruchy klienta master
				while((ret_val = game_processor(board, buf[1], buf[2], (char)2, (char)0)) == 0)		//musimy odpytywac az master wykona odpowiedni ruch
				{

					convert_board_to_send(board, buf);
					buf[0] = GAME_HEAD;
					buf[1023] = 1;				//ustawiamy znacznik ze zly ruch
					write(cdesk,buf, sizeof(buf));		//wysylamy plansze do mastera
					clean_buf(buf);
					printf("\nczeka na master bo ten wykonal zly ruch\n");
					read(cdesk, buf, sizeof(buf));		//oczekujemy na odpowiedz od mastera

					if(buf[0] != GAME_HEAD)			//wystapil blad w kliencie master
					{
						msg_s.content[0] = ERR_TH;
						msgsnd(msgq_id, &msg_s, msg_size,0);
					}

				}		//wyjscie z petli oznacza ze slave wykonal w koncu dobry ruch

				if(ret_val > 0)									//jezeli jest dozwolony to ruch wykonuje komp
				{
					game_processor(board, msg_r.content[1], msg_r.content[2], (char)2, (char)1);	//zatwierdzenie ruchu
					printf("zakceptowal ruch mastera\n");

					if ((isBlocked(board, (char)1))&&(isBlocked(board, (char)2)))		//koniec gry gracze zablokowani
					{
						printf("\nNIE\n");

						msg_s.content[0] = END_TH;
						buf[0] = END_HEAD;

						int master_point = who_is_winner(board, 2);
						int slave_point = who_is_winner(board, 1);

						if(master_point > slave_point)
						{
							msg_s.content[1] = 0;
							buf[1] = 1;
						}
						else
						{
							msg_s.content[1] = 1;
							buf[1] = 0;
						}

						write(cdesk,buf, sizeof(buf));				//wysylamy informacje o koncu rozgrywki do mastera
						msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy informacje o koncu rozgrywki do watku slave


						return;				//konczymy rozgrywke

					}
				}

				convert_board_to_send(board, buf);
				buf[0] = GAME_HEAD;
				buf[1023] = 0;
				printf("\n#wysyla plansze do master\n");
				write(cdesk,buf, sizeof(buf));			//wysylamy plansze do klienta master z potwierdzeniem ruchu bedzie on teraz czekal

				convert_board_to_send(board,msg_s.content);
				msg_s.content[0] = GAME_TH;
				msg_s.content[1023] = 1;

				printf("\n#wysyla plansze do $\n");
				msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy plansze do watku slave ze stanem planszy


			break;
			}

			case CHAT_HEAD:						//klient master wyslal wiadomosc czat
			{
				msg_s.content[0] = CHAT_TH;
				for(i = 1; i < BUF_SIZE ; i++)
				{
					msg_s.content[i] = buf[i];
				}
				ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy wiadomosc do watku o ruchu klienta
				if(ret_val == -1)
				{
					perror("blad podczas wysylania komunikatu");
					buf[0] = ERR_HEAD;
					write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
					return;
				}

			break;
			}

			case ERR_HEAD:
			{
				msg_s.content[0] = ERR_TH;
				ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy wiadomosc do watku o ruchu klienta

				return;				//i konczy dzialanie - wyjscie z while

			break;
			}

			default:
			{
				printf("niepoprawny format wiadomosci wyslany przez klienta master\n");
				return;
			break;
			}

		}//koniec switcha analizujacego odpowiedz klienta master
		*/
	}//while
}

void join_to_mlp_game(int cdesk)	//gra multiplayer - gracz dolaczajacy sie do rozgrywki		//##TU STWORZYC LOGIKE GRY I OBSLUGE CZATU
{
	if(DSP_MSG == 0)
	{
		printf("join_to_mlp_game\n");
	}

	int flag=1;						//informuje czy czekac na odpowiedz klienta
	char buf[BUF_SIZE];
	char tmp_board[BOARD_SIZE][BOARD_SIZE];			//tymczasowa plansza tylko do pierwszego ruchu
	int i=0;
	struct message msg_s, msg_r;				//struktura wiadomosci do wysylania i odbierania
	int neighbour_pid = get_neighbour_pid(getpid());	//pid osoby z ktora gramy
	if(neighbour_pid == -1)
	{
		buf[0] = ERR_HEAD;
		write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
		return;
	}
	msg_s.type = neighbour_pid;				//ustawiamy typ wysylanej wiadomosci wiadomosci na pid sasiada
	msg_r.type = getpid();					//odbieramy wiadomosci takiego typu jaki jest nasz pid

	if(DSP_MSG == 0)
	{
		printf("\nslave pidy %d %d\n", neighbour_pid, getpid());
	}

	clean_buf(buf);
	prepare_board(tmp_board);

	game_processor(tmp_board,(char)-1,(char)-1,(char)1,(char)0);	//przygotowanie planszy
	convert_board_to_send(tmp_board, buf);

	buf[0] = GAME_HEAD;
	buf[1023] = 1;

	write(cdesk, buf,sizeof(buf));				//wysyla plansze do klienta slave z poczatkowym stanem gry

	flag = 1;
	while(1)
	{

		do
		{
			clean_buf(buf);
			//printf("start\n");
			read(cdesk, buf, sizeof(buf));
			//printf("stop %d\n", buf[0]);
			if(buf[0] == ERR_HEAD)			//klient sie rozlaczyl
			{

				//printf("join rozlaczony\n");

				msg_s.content[0] = ERR_TH;
				msgsnd(msgq_id, &msg_s, msg_size,0);

				return;				//wychodzimy z f-cji zeby zakonczyl sie poprawnie watek
			}

			msg_s.content[0] = GAME_TH;
			msg_s.content[1] = buf[1];	//pozycja x
			msg_s.content[2] = buf[2];	//pozycja y


			msgsnd(msgq_id, &msg_s, msg_size,0);


			msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);

			if(msg_r.content[0] == ERR_TH)
			{
				buf[0] = END_HEAD;
				buf[1] = 1;
				write(cdesk,buf, sizeof(buf));
				return;				//wychodzimy z f-cji aby poprawnie zakonczyl sie watek
			}

			if(msg_r.content[0] == END_TH)
			{
				clean_buf(buf);
				buf[0] = END_HEAD;
				buf[1] = msg_r.content[1];
				write(cdesk,buf, sizeof(buf));
				return;
			}


			if(msg_r.content[1023] == 1)
				flag = 1;
			else
				flag = 0;

			for(i=1 ; i < BUF_SIZE ; i++)
			{
				buf[i] = msg_r.content[i];
			}

			buf[0] = GAME_HEAD;
			if(flag == 1)
				buf[1023] = 1;
			else
				buf[1023] = 0;

			write(cdesk,buf, sizeof(buf));
		}
		while(flag == 1);




		do
		{
			msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);

			if(msg_r.content[0] == ERR_TH)
			{
				buf[0] = END_HEAD;
				buf[1] = 1;
				write(cdesk,buf, sizeof(buf));
				return;				//wychodzimy z f-cji aby poprawnie zakonczyl sie watek
			}

			if(msg_r.content[0] == END_TH)
			{
				clean_buf(buf);
				buf[0] = END_HEAD;
				buf[1] = msg_r.content[1];
				write(cdesk,buf, sizeof(buf));
				return;
			}

			for(i=1 ; i < BUF_SIZE ; i++)
			{
				buf[i] = msg_r.content[i];
			}

			buf[0] = GAME_HEAD;
			if(msg_r.content[1023] == 0)
				flag = 0;
			else
				flag = 1;

			if(flag == 0)
				buf[1023] = 0;
			else
				buf[1023] = 1;

			write(cdesk,buf, sizeof(buf));
		}
		while(flag == 0);

		/*
		clean_buf(buf);


		if(flag == 1)
		{
			printf("\n$ czeka na odp klienta slave\n");

			read(cdesk, buf, sizeof(buf));			//czeka na ruch klienta slave

			printf("\n.\n");


			//analiza tego co przysyla klient i odpowiednia reakcja
			switch (buf[0])
			{
				case GAME_HEAD:				//klient slave wykonal ruch i wysylana jest informacja do watku master
				{
					msg_s.content[0] = GAME_TH;	//ustawiamy naglowek do watku master
					msg_s.content[1] = buf[1];	//pozycja x
					msg_s.content[2] = buf[2];	//pozycja y


					ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy wiadomosc do watku master o ruchu klienta slave
					if(ret_val == -1)
					{
						perror("blad podczas wysylania komunikatu");
						buf[0] = ERR_HEAD;
						write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
						return;
					}

				break;
				}

				case END_HEAD:
				{
					//nie ma topscora bo to multiplayer
				break;
				}

				case CHAT_HEAD:							//wiadomosc czat
				{

					msg_s.content[0] = CHAT_TH;
					for(i = 1; i < BUF_SIZE ; i++)
					{
						msg_s.content[i] = buf[i];
					}
					ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy wiadomosc czat do watku master
					if(ret_val == -1)
					{
						perror("blad podczas wysylania komunikatu");
						buf[0] = ERR_HEAD;
						write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
						return;
					}

				break;
				}

				case ERR_HEAD:							//wystapil blad
				{
					msg_s.content[0] = ERR_TH;
					ret_val = msgsnd(msgq_id, &msg_s, msg_size,0);		//wysylamy wiadomosc o bledze do watku master

					return;				//i konczy dzialanie - wyjscie z while

				break;
				}

				default:
				{
					printf("bledna wiadomosc od klienta\n");
					return;
				}


			}	//koniec analizy tego co przyslal klient slave

		}




		clean_buf(buf);
		//analiza tego co przysyla watek master i odpowiednia reakcja
		printf("\n$ oczekuje na odp mastera\n");

		ret_val = msgrcv(msgq_id, &msg_r, msg_size, getpid(),0);			//oczekuje na odpowiedz na plansze z ruchem przeciwnika od watku master
		printf("\n$taka odpowiedz: zerowy: %d ost: %d\n", msg_r.content[0], msg_r.content[1023]);
		if(ret_val == -1)
		{
			perror("blad podczas odbierania komunikatu");
			buf[0] = ERR_HEAD;
			write(cdesk,buf, sizeof(buf));
			return;
		}
		switch(msg_r.content[0])
		{
			case  GAME_TH:					//otrzymalismy plansze od sasiadniego watku
			{

				if(msg_r.content[1023] == 0)
				{
					flag = 0;			//plansza potwierdzajaca klient nic nie bedzie wysylac
					//printf("\n$dostal 0\n");
				}
				else
				{
					flag = 1;			//zly ruch klient bedzie wysylal
					//printf("\n$dostal 1\n");
				}

				printf("\n$ do slava\n");
				for(i = 1; i < BUF_SIZE ; i++)
				{
					buf[i] = msg_r.content[i];
					printf("%d ",buf[i]);
				}
				buf[0] = GAME_HEAD;


				if(flag == 0)
				{
					buf[1023] = 0;
				}
				else
					buf[1023] = 1;
				printf("\n$ wysyla to samo to slave: zero: %d ost: %d", buf[0], buf[1023]);
				write(cdesk,buf, sizeof(buf));			//wysylamy plansze do klienta slave

			break;
			}

			case REPEAD_TH:					//oznacza to ze wykonany ruch klienta slave byl nieprawidlowy
			{

				for(i = 1; i < BUF_SIZE ; i++)
				{
					buf[i] = msg_r.content[i];
				}
				buf[0] = GAME_HEAD;
				write(cdesk,buf, sizeof(buf));			//wysylamy plansze do klienta slave

			break;
			}

			case END_TH:					//otrzymalismy wiadomosc o koncu rozgrywki
			{
				buf[0] = END_HEAD;
				for(i = 1; i < BUF_SIZE ; i++)
				{
					buf[i] = msg_r.content[i];
				}
				write(cdesk,buf, sizeof(buf));		//wysylamy do klienta slave wiadomosc o koncu rozgrywki

				return;					//po dostaniu tej wiadomosci konczymy rozgrywke

			break;
			}


			case CHAT_TH:					//otrzymalismy wiadomosc czat
			{

				buf[0] = CHAT_HEAD;
				for(i = 1; i < BUF_SIZE ; i++)
				{
					buf[i] = msg_r.content[i];
				}
				write(cdesk,buf, sizeof(buf));			//wysylamy tresc wiadomosci czat do klienta

			break;
			}


			case ERR_TH:
			{
				buf[0] = ERR_HEAD;
				write(cdesk,buf, sizeof(buf));
				return;						//w przypadku wystapienia bledu wychodzi

			break;
			}

			default:
			{
				printf("bledna wiadomosc od watku master\n");
				return;
			}

		}		//koniec analizy odpowiedzi watku master

*/
	}	//while
}

void multiplayer(int cdesk)		//f-cja dzialajaca w watku; gra 2-osobowa
{
	if(DSP_MSG == 0)
	{
		printf("multiplayer\n");
	}
	char buf[BUF_SIZE];
	int ret_val=0;

	ret_val = save_mlp_id(getpid());//zapisuje pid do pamieci wspoldzielonej i uzyskuje odpowiedz czy bedzie zakladal rozgrywke czy sie dolaczal
	if(ret_val == -1)		//oznacza to ze w systemie jest juz za duzo uzytkownikow
	{
		buf[0] = ERR_HEAD;
		write(cdesk,buf, sizeof(buf));		//##USTALIC TRESC KOMUNIKATU
		//nie musimy robic remove_mlp_id() bo nie udalo sie wogle wpisac do pamieci
	}
	else
	{


		if(ret_val == 0)	//gracz zaklada rozgrywke
		{
			create_mlp_game(cdesk);
		}
		else			//gracz przylacza sie do rozgrywki
		{
			join_to_mlp_game(cdesk);
		}

		remove_mlp_id(getpid());	//po zakonczonej rozgrywce gracz usuwa swoj pid z pamieci wspoldzielonej odpowiedzialnej za rozgrywke multiplayer
	}

}

void validate_users(int cdesk, int pid)		//sprawdza czy uzytkownicy nie sa zbyt dlugo przylaczeni
{
	struct validate* v_ptr;
	time_t seconds;
	int* shm_addr;
	int* shm_addr_soc;
	int time_now = 0;
	int i=0, j=0, k=0;
	int flag = 0;

	seconds = (time_t)time;
	time_now =  seconds/3600;

	if(DSP_MSG == 0)
	{
		printf ("%ld hours since January 1, 1970", seconds/3600);
	}

	v_ptr = (struct validate*)shmat(shmid_val, NULL, 0);
	if(v_ptr == NULL)
	{
		perror("Blad podczas przylaczania pamieci wspoldzielonej w validate_users");
		exit(-1);
	}


	semop(semid_usr, &Pval, 1);		//opuszczanie semafor

	for(i=0 ; i < MAX_USR ; i++)			//dodawanie nowego uzytkownika
	{

		if(v_ptr[i].pid_thr == 0)		//odnajduje wolna pozycja w tablicy
		{
			v_ptr[i].dsc_thr = cdesk;
			v_ptr[i].log_time = time_now;
			v_ptr[i].pid_thr = pid;
		}
	}

	shm_addr = shmat(shmid_pid, NULL, 0);
	semop(semid_usr, &Ppid, 1);						//walidacja uzytownikow w systemie
	for(i = 0 ; i < MAX_USR ;  i++)			//przechodzi po calej strukturze validate
	{
		flag=0;
		for(j = 0 ; j < MAX_USR ; j++)		//sprawdzajac czy jest aktualna (porownuje z tablica z pidami)
		{
			if(shm_addr[j] == v_ptr[i].pid_thr)		//jezeli pid zostal odnaleziony
			{
				int chck_time=0;
				chck_time = v_ptr[i].log_time - time_now;		//sprawdzamy czy nie zostal przekroczony czas
				if(chck_time > TOLERATION)				//jezeli czas zostal przekroczony nalezy usunac watek z systemu
				{
					kill(shm_addr[j],9);			//zabijamy watek
					shm_addr[j] = 0;			//usuwamy go z pamieci wspoldzielonej z pidami

					decrement_number_of_users();		//zmniejszamy liczbe uzytkownikow w systemie;


					shm_addr_soc = shmat(shmid_soc, NULL, 0);	//przylaczamy pamiec z gniazdkami nie musimy podnosic semafora bo tylko czytamy
					for(k = 1 ; k < SOC_NMB ; k++)		//dla bhp zamykamy jeszcze gniazdka nie musimy sprawdzac 1 bo to jest od glownego watku
					{
						if(shm_addr_soc[k] == v_ptr[i].dsc_thr)
						{
							chck_time = v_ptr[i].log_time - time_now;
							if(chck_time > TOLERATION)
							{
								close_dsc(shm_addr_soc[k]);					//zamykamy gniazdko
								remove_soc_dsc(shm_addr_soc[k]);	//usuwamy gniazdko z pamieci wspoldzielonej
							}
						}

					}
					shmdt(shm_addr_soc);			//odlaczamy pamiec z gniazdkami

				}
				flag = 1;				//oznacza ze watek istnieje - nie nalezy go wymazywac z pamieci
			}
		}

		if(flag == 0)		//watek juz zostal usuniety i nalezy go teraz usunac z pamieci zeby byla ona aktualna
		{
			v_ptr[i].dsc_thr = 0;
			v_ptr[i].log_time = 0;
			v_ptr[i].pid_thr = 0;
		}

	}
	semop(semid_usr, &Vpid, 1);		//podnoszenie semafora od pidow
	//odlaczenie pamieci z pidami


	semop(semid_usr, &Vval, 1);		//podnoszenie samafora
	//printf("podniosl semafor\n");
	shmdt(v_ptr);

}

void run()
{
	fd_set fsMask, fsRmask, fsWmask;	//The fd_set data type represents file descriptor sets for the select function. It is actually a bit array.
	int nFound, nMaxfd,nFd,nFoo=1;
	static struct timeval tTimeout;

	struct sockaddr_in addr, caddr;	       //struktura addr przechowuje informacje serwera; caddr klienta
	int desk, cdesk;		//desk - deskryptor gniazda serwera, cdesk - podłączającego się klienta
	socklen_t len = sizeof(caddr);
	addr.sin_family=PF_INET;
	addr.sin_port=PORT;
	addr.sin_addr.s_addr=INADDR_ANY;
	int ret_val=0;


	desk = socket(PF_INET,SOCK_STREAM,0);	//tworzy gniazdo i zwraca deskryptor do niego
	if(desk < 0)
	{
		perror("Blad podczas uzywania f-cji socket");
		exit(-1);
	}
	ret_val = add_soc_dsc(desk);
	if(ret_val == -1)
	{
		if(DSP_MSG == 0)
		{
			printf("Blad podczas zapamietywania deskryptora\n");
		}
		//exit(-1);
	}

	setsockopt(desk, SOL_SOCKET, SO_REUSEADDR, (char*)&nFoo, sizeof(nFoo));		//uwaga: nawet SO_REUSEADDR nie pomoże jeśli jakies gniazdo nasłuchuje (LISTEN) na danym adresie

	ret_val = bind(desk,(const struct sockaddr*)&addr,sizeof(addr));		//wiaze utworzone wczesniej gniazdo z adresem maszyny
	if(ret_val < 0 )
	{
		perror("Blad w f-cji bind");
		exit(-1);
	}

	ret_val = listen(desk,BACKLOG);		//przygotowuje utworzone gniazdo do odbierania zgłoszeń
	if(ret_val < 0)
	{
		perror("blad w f-cji listen");
		exit(-1);
	}
						//makra pomocnicze dla f-cji select:
	FD_ZERO(&fsMask);			//zerujemy zbior deskryptorow fsMask
	FD_ZERO(&fsRmask);			//zerujemy zbior deskryptorow fsRmask
	nMaxfd = desk;				//maksymalna 	WARTOSC deskryptora

	while(1)
	{



		FD_SET(desk, &fsRmask);		//dodajemy deskryptor do zbioru deskryptorow musi byc w petli bo select() zeruje pozniej te ustawienia
		fsWmask = fsMask;		//UWAGA  fsWmask sa fizycznie tablicami stad tutaj przepisujemy WSKAZNIK !!!
		tTimeout.tv_sec = TIMEOUT;	//ustawiamy timeout musimy to robic przed kazdym wywolaniem select()
		tTimeout.tv_usec = 0;

		//ret_val = check_access(); //II SPOSOB NA OGRANICZENIE LICZBY UZYTKOWNIKOW while check_number_of_users() == -1 wait

		nFound = select(nMaxfd + 1, &fsRmask, &fsWmask, (fd_set*) 0, &tTimeout);   //Funkcja select umożliwia nadzorowanie zbioru deskryptorów pod względem możliwości odczytu, zapisu bądź wystąpienia sytuacji wyjątkowych. Zwraca ilosc deskryptorow, ktore zmienily stan albo 0 - timeout albo liczbe ujemna - blad

		if(nFound<0)
		{
			perror("blad w f-cji select");
			exit(-1);
		}
		if(nFound == 0)		//oznacza, że wystąpił  timeout
		{
			//printf("timeout\n");
			fflush(stdout);
		}
		if(FD_ISSET(desk, &fsRmask))	//sprawdzana jest przynaleznosc deskryptora desk do zbioru deskryptorow fsRmask
		{
			//accept (int socket, struct sockaddr *addr, size_t *length_ptr)
			cdesk=accept(desk, (struct sockaddr*) &caddr, &len);	//pobiera zgloszenie z kolejki (dziala w sposob blokujacy ale i tak mamy selecta)
			if(cdesk < 0)
			{
				perror("blad w f-cji accept");
				exit(-1);
			}
/*
			ret_val = add_soc_dsc(cdesk);		//zapamietuje deskyptor zeby w razie zakonczenia pracy serwera mozna go bylo zamknac
			if(ret_val == -1)
			{
				if(DSP_MSG == 0)
				{
					printf("Blad podczas zapamietywania deskryptora\n");
				}
				//exit(-1);
			}
*/

			FD_SET(cdesk, &fsMask);	//ustawiamy deskryptor cdesk w zbiorze deskryptorow fdMask (uwaga wykonanna byla instrukcja fsWmask = fsMask stad ustawi sie fsWmask (gdyby zamiast tej instrukcji bylo FD_SET(cdesk, &fsWmask); tez by dzialalo
			if (cdesk > nMaxfd)	//jezeli nowy deskryptor ma wieksza wartosc niz obecnie najwiekszy
				nMaxfd = cdesk;   	 //to zapamietujemy go w nMaxfd
		}
		for (nFd = 0; nFd <= nMaxfd; nFd++)	//sprawdzamy po wszystkich deskryptorach czy korys przynalezy do zbioru fsWmask
		{
			if(FD_ISSET(nFd, &fsWmask))	//przez to ze zrobilismy FD_SET(cdesk, &fsMask) ustawilo sie rowniez w fsWmask bo byla wczesniej wykonanana instrukcja fsWmask = fsMask sa ta tablice wiec pokazuja na to samo miejsce w pamieci
			{

				int length=0;
				char buf[BUF_SIZE];
				length=read(cdesk,buf,sizeof(buf));
				if(DSP_MSG == 0)
				{
					printf("\ndane z logowania:\n");
				}
				write(1, buf, length);
				if(DSP_MSG == 0)
				{
					printf("=====");
				}


				ret_val = login(cdesk, buf, desk);		//logowanie sie jeszcze przed utworzeniem watku dla klienta
				if(ret_val == 0)			//jezeli logowanie powiodlo sie to stworzony zostanie watek obslugujacy danego klienta
				{
					add_soc_dsc(cdesk);		//zapamietuje deskyptor zeby w razie zakonczenia pracy serwera mozna go bylo zamknac

					if(buf[0] == LOG_HEAD)			//przez watek glowny obslugiwane tylko wiadomosci typu 0
					{
						if((ret_val = fork()) == 0)
						{

							validate_users(cdesk, getpid());
							increment_number_of_users();	//po udanym zalogowaniu zwiekszana jest liczba liczba uzytkownikow
							if(buf[1] == 0)		//tryb rozgrywki single player
							{
								singleplayer(cdesk);
							}
							else			//tryb rozgrywki multiplayer
							{
								multiplayer(cdesk);
							}

							decrement_number_of_users();	//po zakonczeniu rozgrywki dekrementowana jest liczba uzytkownikow
							remove_soc_dsc(cdesk);
							close(cdesk);			//zamykamy gniazdo od naszego accepta
							if(DSP_MSG == 0)
							{
								printf("koniec watku\n");
							}
							exit(0);
						}
						else
						{
							if(ret_val == -1)
							{
								perror("blad podczas tworzenia watku");
								remove_soc_dsc(cdesk);
								close(cdesk);
								exit(-1);                       //###ZMIENIC exita na wysylanie komunikatu o bledzie do uzytkownika
							}
						}
					}
					else	//jezeli klient przyslal wiadomosc w zlej formie
					{
						if(DSP_MSG == 0)
						{
							printf("klient przyslal zla wiadomosc\n");
						}
						remove_soc_dsc(cdesk);
						close(cdesk);
					}

				}
				else	//jezeli logowanie nie powiodlo sie
				{

					if(ret_val == 2)
					{

						if(DSP_MSG == 0)
						{
							printf("adnim\n");
						}
						/*FD_ZERO(&fsMask);			//zerujemy zbior deskryptorow fsMask
						FD_ZERO(&fsRmask);
						*/
					}

					remove_soc_dsc(cdesk);
					close(cdesk);
				}
				//remove_soc_dsc(cdesk);	//###UWAGA ODKOMENTOWAC W PRZYPADKU GDY DESKRYPTORY TRZEBA ZAMYKAC I W WATKU POTOMNYM I W GLOWNYM
				FD_CLR(nFd, &fsMask);		//usuwamy deskryptor ze zbioru deskryptorow
				//close(cdesk);	//### zamykamy dyskryptor klienta UWAGA ODKOMENTOWAC W PRZYPADKU GDY DESKRYPTORY TRZEBA ZAMYKAC I W WATKU POTOMNYM I W GLOWNYM
			}
		}

	}
	remove_soc_dsc(desk);
	close(desk);

	if(DSP_MSG == 0)
	{
		printf("koniec\n");
	}

}

void clean(int sig)			//funkcja sprzatajaca: zabija wszystkie watki uzytkownikow, usuwa pamiec wspoldzielona i semafory
{
	if(DSP_MSG == 0)
	{
		printf("clean\n");
	}

	save_top_score();
	close_all_soc_dsc();
	kill_all_threads();
	remove_queue();
	remove_shared_memory();
	remove_semafor();

	exit(0);			//tutaj exit czyli po nacisnieciu ctrl+c automatyczne zakonczenie pracy serwera
}

//===============================================================================================

void menu()     //dziala w watku odpowiada za konfiguracje serwera
{
        int choice=0;
        char text[255];
/*
        char* text = (char*)malloc(sizeof(char)*255);
        if(text == NULL)
        {

        }
        free(text);
*/

        while(1)
        {
                choice = -1;
                fflush(stdin);
                scanf("%s", text);


                if(strcmp(text, "run") == 0)
                {
                        choice = 0;
                }
                else
                        if(strcmp(text, "stop") == 0)
                        {
                                choice = 1;
                        }


                switch (choice)
                {
                        case 0:
                                printf("start\n");
                        break;

                        case 1:
                        {
                                printf("stop\n");

                                //close_all_soc_dsc();
                                //kill_all_threads();
                                remove_queue();
                                remove_shared_memory();
                                remove_semafor();

                                break;
                        }
                        break;

                        default:
                                printf("command not found\n");
                        break;


                }
        }

}

//===============================================================================================

 int main(int argc, char* argv[])
 {

        void (*func_pointer)(int);
        func_pointer = clean;
        signal(2, func_pointer);


	create_queue();
	create_shared_memory();
	create_semafor();
	get_top_score();

	run();


	return 1;

 }

